import time
from math import floor

from epics import caget, caput

import iexcode.instruments.cfg as iex

from iexcode.instruments.utilities import print_warning_message
from iexcode.instruments.scalers import scaler_cts


##############################################################################################################
##############################            Motor Class        ##############################
##############################################################################################################
class Motors:
    """ 
    short hand to move motors in endstation
    usage motors = Motor('ARPES','ARPES_motor_dictionary')
    """

    def __init__(self,endstation_name,motor_dictionary,physical_motors,
    psuedo_motors):
        self.endstation_name = endstation_name
        self._motor_dictionary = motor_dictionary
        self.physical_motors = physical_motors
        self.pseudo_motors = psuedo_motors
           
    def info(self):
        print("physical motors:",self.physical_motors)
        print("pseudo motors:",self.pseudo_motors)
    
    def get(self,name,verbose=False):   
        """
        get current position of motor name = "x","y"...
        """   

        try:
            rbv = caget(self._motor_dictionary[name][0])
            if verbose:
                print('current position: '+name+" = "+str(rbv))
            return rbv
        except:
            print("Not a valid motor name") 
    
    def reset(self,name):
        """
        Reset motor if stuck in 'moving'
        """
        pv=self._motor_dictionary[name][2]
        caput(pv,'Stop')
        time.sleep(1)
        caput(pv,'Go')
        
    def status(self):
        """
        if status == 0 then moving
        """
        status = 0
        for motor in self._motor_dictionary.keys():
            status *= caget(self._motor_dictionary[motor][3]+".DMOV")
        return status

    def move(self,name,val,**kwargs):
        """
        Moves a motor in the endstation based on common name, not PV name
        
        name: motor name => 'x','y','z','th','chi','phi' ...
        val: absolute position if relative=False, delta if relative=True
        
        **kwargs:
            relative: move mode => True/False; default = False (absolute)
            wait: wait for move completion before moving on => True/False; default=True
            verbose: print old and new motor position => True/False; default=False


        Previously: Move_ARPES_Motor or Move_ARPES_Motor or motor_move_vs_Branch
        """
        kwargs.setdefault('relative',False)
        kwargs.setdefault('wait',True)
        kwargs.setdefault('verbose',False)

        try:
            rbv_pv,val_pv,sgm_pv,pv = self._motor_dictionary[name]
            old_position = caget(rbv_pv)
            if kwargs['relative']:
                val = old_position + val
            
            caput(val_pv,val,wait=kwargs['wait'],timeout=18000)
            
            new_position = caget(rbv_pv)
            if kwargs['verbose']:
                print('old: '+name+" = "+str(old_position))
                print('new: '+name+" = "+str(new_position))
        except:
            print(name+" is not a valid motor name")



    def dmove(self,name,val,**kwargs):
        """
        relative move of a motor in the endstation based on common name, not PV name
        name = x,y,z,th,chi,phi ...

        **kwargs:
            wait: wait for move completion before moving on => True/False; default=True
            verbose: print old and new motor position => True/False; default=False

        Previously: motor_umove_vs_Branch
        """
        kwargs.setdefault('wait',True)
        kwargs.setdefault('verbose',False)

        self.move(name,val,relative=True,wait=kwargs['wait'],verbose=kwargs['verbose'])

    def mprint(self):
        """
        prints current position of the physical motors
        """
        position_list = []
        for motor in self.physical_motors:
            position_list.append(self.get(motor,verbose=False))
        return position_list

    def mvsample(self,position_list,verbose=True):
        """
        moves the sample to the position sepcified by position_list
        position_list = ['description',x,y,z,th,chi,phi]

        Previously: sample
        """
        if not isinstance(position_list[0],str):
            sample_name = ''
        else:
            sample_name = position_list[0]
            position_list = position_list[1:]

        motor_list = self.physical_motors
        #debug
        print('motor_list=',motor_list)
        print('position_list=',position_list)
        for i,motor in enumerate(motor_list):
            self.move(motor,position_list[i])

        if verbose:
            print("Sample now @ "+sample_name)

    def scan(self,name,start,stop,step,**kwargs):
        """
        scans a motor
        
        **kwargs:
            relative: move mode => True/False; default = False (absolute)
            scan_dim: 1  (default)
            execute: True/False to start the scan => True (default)

            for Kappa only:
                cts: scaler integration time in seconds (default = 0.1 s)
                mpa: True/False to sum mpa acquisitions
            
            Previously Scan_ARPES_motor / Scan_Kappa_motor
        """
        kwargs.setdefault('relative',False)
        kwargs.setdefault('scan_dim',1)
        kwargs.setdefault('execute',True)
       
        #for kappa only
        kwargs.setdefault('cts',0.1)
        kwargs.setdefault('mpa',False)
        motor_pvs = self._motor_dictionary.get(name)
        if motor_pvs:
            rbv_pv,val_pv,sgm_pv,pv = motor_pvs
        else:#here
            print_warning_message('motor = "'+str(name)+'" is not defined for '+iex.BL.endstation)
            return

        if kwargs['relative']:
            current_value = caget(rbv_pv)
            abs_start = round(current_value + start,3)
            abs_stop  = round(current_value + stop,3)
            print("start, stop, step = "+str(abs_start)+", "+str(abs_stop)
            +", "+str(step))
        else:
            abs_start = start
            abs_stop = stop
        
        scaler_cts(kwargs['cts'],verbose=True)
        iex.BL.mda.fillin(val_pv,rbv_pv,abs_start,abs_stop,step,**kwargs)
        
        if kwargs['execute']:
            iex.BL.mda.go(**kwargs)


    def scan_2D(self,inner_loop_list,outer_loop_list,**kwargs):
        """
        Fills in a the Scan Record for a 2D scan (Mesh), does NOT press go
        InnerMotorList=[name1,start1,stop1,step1] #scanDIM=2
        OuterMotorList=[name2,start2,stop2,step2] #scanDIM=3
            name = 'x'/'y'/'z'/'th'...    
        
        **kwargs:
            relative: scab => True/False; default = False (absolute)
            outer_scan_dim: 2  (default)
            execute: True/False to start the scan => True (default)
            Snake; coming soon


            for Kappa only:
                cts: scaler integration time in seconds (default = 0.1 s)
                mpa: True/False to sum mpa acquisitions

        """
        kwargs.setdefault('mode','absolute')
        kwargs.setdefault('outer_scan_dim',2)
        kwargs.setdefault('snake',False)
        kwargs.setdefault('execute',True)
        #for kappa only
        kwargs.setdefault('cts',0.1)
        kwargs.setdefault('mpa',False)

        from iexcode.instruments.IEX_BL_config import BL

        rbv_pv,val_pv,sgm_pv,pv = self._motor_dictionary[inner_loop_list[0]]
        inner_loop_list.insert(0,rbv_pv)
        inner_loop_list.insert(0,val_pv)
        
        rbv_pv,val_pv,sgm_pv,pv = self._motor_dictionary[outer_loop_list[0]]
        outer_loop_list.insert(0,rbv_pv)
        outer_loop_list.insert(0,val_pv)

        if kwargs["mode"] == "relative":
            current_value1 = caget(inner_loop_list[1])
            inner_loop_list[2]=round(current_value1+inner_loop_list[2],3)
            inner_loop_list[3]=round(current_value1+inner_loop_list[3],3)

            current_value2 = caget(outer_loop_list[1])
            outer_loop_list[2]=round(current_value1+outer_loop_list[2],3)
            outer_loop_list[3]=round(current_value1+outer_loop_list[3],3)

        scaler_cts(kwargs['cts'],verbose=True,**kwargs)
        iex.BL.mda.fillin_2D(inner_loop_list,outer_loop_list,
        outer_scan_dim=kwargs['outer_scan_dim'],**kwargs)

        if kwargs['execute']:
            iex.BL.mda.go(**kwargs)




##############################################################################################################
##############################            Motor Short Names         ##############################
##############################################################################################################
### x ###
def mvx(val,**kwargs):
    """ moves x motor in the endstation"""
    iex.BL.Motors.move('x',val,**kwargs)

def dmvx(val):
    """ relative x motor in the endstation"""
    iex.BL.Motors.move('x',val,relative=True)

def scanx(start,stop,step,**kwargs):
    """ scans the x motor in the endstation"""
    iex.BL.Motors.scan("x",start,stop,step,**kwargs)

def dscanx(start,stop,step):
    """relative scans of the x motor in the endstation """
    iex.BL.Motors.scan("x",start,stop,step,relative=True)

### y ###
def mvy(val,**kwargs):
    """ moves y motor in the endstation"""
    iex.BL.Motors.move('y',val,**kwargs)

def dmvy(val):
    """ relative y motor in the endstation"""
    iex.BL.Motors.move('y',val,relative=True)

def scany(start,stop,step,**kwargs):
    """ scans the y motor in the endstation"""
    iex.BL.Motors.scan("y",start,stop,step,**kwargs)

def dscany(start,stop,step):
    """relative scans of the y motor in the endstation """
    iex.BL.Motors.scan("y",start,stop,step,relative=True)

### z ###
def mvz(val,**kwargs):
    """ moves z motor in the endstation"""
    iex.BL.Motors.move('z',val,**kwargs)

def dmvz(val):
    """ relative z motor in the endstation"""
    iex.BL.Motors.move('z',val,relative=True)

def scanz(start,stop,step,**kwargs):
    """ scans the z motor in the endstation"""
    iex.BL.Motors.scan("z",start,stop,step,**kwargs)

def dscanz(start,stop,step):
    """relative scans of the z motor in the endstation """
    iex.BL.Motors.scan("z",start,stop,step,relative=True)

### kth ###
def mvkth(val,**kwargs):
    """ moves kth motor in the endstation"""
    iex.BL.Motors.move('kth',val,**kwargs)

def dmvkth(val):
    """ relative kth motor in the endstation"""
    iex.BL.Motors.move('kth',val,relative=True)

def scankth(start,stop,step,**kwargs):
    """ scans the kth motor in the endstation"""
    iex.BL.Motors.scan("kth",start,stop,step,**kwargs)

def dscankth(start,stop,step):
    """relative scans of the kth motor in the endstation """
    iex.BL.Motors.scan("kth",start,stop,step,relative=True)

### kphi ###
def mvkphi(val,**kwargs):
    """ moves kphi motor in the endstation"""
    iex.BL.Motors.move('kphi',val,**kwargs)

def dmvkphi(val):
    """ relative kphi motor in the endstation"""
    iex.BL.Motors.move('kphi',val,relative=True)

def scankphi(start,stop,step,**kwargs):
    """ scans the kphi motor in the endstation"""
    iex.BL.Motors.scan("kphi",start,stop,step,**kwargs)

def dscankphi(start,stop,step):
    """relative scans of the kphi motor in the endstation """
    iex.BL.Motors.scan("kphi",start,stop,step,relative=True)   

### kap ###
def mvkap(val,**kwargs):
    """ moves kap motor in the endstation"""
    iex.BL.Motors.move('kap',val,**kwargs)

def dmvkap(val):
    """ relative kap motor in the endstation"""
    iex.BL.Motors.move('kap',val,relative=True)

def scankap(start,stop,step,**kwargs):
    """ scans the kap motor in the endstation"""
    iex.BL.Motors.scan("kap",start,stop,step,**kwargs)

def dscankap(start,stop,step):
    """relative scans of the kap motor in the endstation """
    iex.BL.Motors.scan("kap",start,stop,step,relative=True)

### tth ###
def mvtth(val,**kwargs):
    """ moves tth motor in the endstation"""
    iex.BL.Motors.move('tth',val,**kwargs)

def dmvtth(val):
    """ relative tth motor in the endstation"""
    iex.BL.Motors.move('tth',val,relative=True)

def scantth(start,stop,step,**kwargs):
    """ scans the tth motor in the endstation"""
    iex.BL.Motors.scan("tth",start,stop,step,**kwargs)

def dscantth(start,stop,step):
    """relative scans of the tth motor in the endstation """
    iex.BL.Motors.scan("tth",start,stop,step,relative=True)

### th ###
def mvth(val,**kwargs):
    """ moves th motor in the endstation"""
    iex.BL.Motors.move('th',val,**kwargs)

def dmvth(val):
    """ relative th motor in the endstation"""
    iex.BL.Motors.move('th',val,relative=True)

def scanth(start,stop,step,**kwargs):
    """ scans the th motor in the endstation"""
    iex.BL.Motors.scan("th",start,stop,step,**kwargs)

def dscanth(start,stop,step):
    """relative scans of the th motor in the endstation """
    iex.BL.Motors.scan("th",start,stop,step,relative=True)

### chi ###
def mvchi(val,**kwargs):
    """ moves chi motor in the endstation"""
    iex.BL.Motors.move('chi',val,**kwargs)

def dmvchi(val):
    """ relative chi motor in the endstation"""
    iex.BL.Motors.move('chi',val,relative=True)

def scanchi(start,stop,step,**kwargs):
    """ scans the chi motor in the endstation"""
    iex.BL.Motors.scan("chi",start,stop,step,**kwargs)

def dscanchi(start,stop,step):
    """relative scans of the chi motor in the endstation """
    iex.BL.Motors.scan("chi",start,stop,step,relative=True)

### phi ###
def mvphi(val,**kwargs):
    """ moves phi motor in the endstation"""
    iex.BL.Motors.move('phi',val,**kwargs)

def dmvphi(val):
    """ relative phi motor in the endstation"""
    iex.BL.Motors.move('phi',val,relative=True)

def scanphi(start,stop,step,**kwargs):
    """ scans the phi motor in the endstation"""
    iex.BL.Motors.scan("phi",start,stop,step,**kwargs)

def dscanphi(start,stop,step):
    """relative scans of the phi motor in the endstation """
    iex.BL.Motors.scan("phi",start,stop,step,relative=True)

### uan moves tth and th simultaneously 
def uan(tth,th):
    """ Moves tth and th motors simultaneously in the in the Kappa chamber
    """
    if iex.BL.endstation == 'Kappa':
        #move tth and th
        mvth(th,wait=False)
        mvtth(tth,wait=False)
        sleep(0.2)     

        while True:
            status = iex.BL.Motors.status()
            if status == 0:
                sleep(0.2)
            else:
                tth_RBV=round(iex.BL.Motors.get('tth',verbose=False),3)
                th_RBV=round(iex.BL.Motors.get('th',verbose=False),3)
                print('tth='+str(tth_RBV)+' th='+str(th_RBV))
                break
    else:
        print("not a defined move")
######## mprint and mvsample ###################
def mprint():
    """
    prints current position of the physical motors
    """
    iex.BL.Motors.mprint()

def mvsample(position_list):
    """
    moves the sample to the position sepcified by position_list
    position_list = ['description',x,y,z,th,chi,phi]

    Previously: sample
    """
    iex.BL.Motors.mvsample(position_list)