import time
import numpy as np
from epics import caget,caput, PV


#-----------------------------------------------
#--- Beamline dependent PVs and definitions  ---
#-----------------------------------------------
P="29idcScienta:"
PHV=P+'HV:'
Pcam =P+'basler:cam1:'
savePlugin = P+"HDF1:"
statsPlugin = P+"Stats4:"
dtype = "h5"
#energy scaling
KEpix=0.0000845#*PE
KEwidth=0.1085#*PE


def AllowedEnergyRange(*args):
    """
    KEmin, KEmax for a given PassEnergy, LensMode combination
    *args 
           = PassEnergy, LensMode => returns KEmin,KEmax
           = empty => returns entire table
 
    """
    Table={}
    Table['Transmission']={1:(0,76),2:(0,107),5:(1,237),10:(1,453),20:(3,968),50:(7,6041),100:(21,6000),200:(30,6206),500:(283,6504)}
    Table['Angular']={1:(None,None),2:(1,52),5:(2,131),10:(2,262),20:(10,523),50:(24,1309),100:(34,6105),200:(230,6206),500:(None,None)}
    Table['Angular_01']={1:(None,None),2:(None,None),5:(None,None),10:(None,None),20:(None,None),50:(126,2947),100:(252,1401),200:(844,1261),500:(None,None)}
    Table['Angular_05']={1:(None,None),2:(None,None),5:(None,None),10:(None,None),20:(None,None),50:(None,None),100:(252,1323),200:(1043,1270),500:(None,None)}
    
    if len(args)<1:
        return Table
    
    if len(args)==2:
        try:
            PassEnergy,LensMode=args
            KEmin,KEmax=Table[LensMode][PassEnergy]
            return KEmin,KEmax
        except:
            return None
    else:
        print('Not a valid argument')

#############################################################
########### Scienta python control functions ################
#############################################################

class Scienta:
    """
    Scienta only talks in 'KE' anything related to the binding energy happens outside of these
    controls
    Usage: 
        EA=Scienta()
        EA.get(); cagets all Scienta parameters and returns vars(EA); dictionary of variables
        EA.PE(),EA.KE(),EA.LensMode()
    """
    
    def __init__(self):
        #High Voltage Setting
        self.PHV = PHV
        self._Pcam=Pcam
        self._savePlugin=savePlugin
        self._statsPlugin=statsPlugin
        
        self.dtype=dtype
        self.wk = None

        self.ElementSet = None #HV for XPS, LV for UPS

        self.LensMode = None
        self.PassEnergy = None
        self.KineticEnergy = None
        self.SpectraMode = None

        self.connected = None

        #Camera Settings
        self.Frames = None
        
        pv = PV(self.PHV+"WorkFunction")
        if pv.connect():
            self.get()
            self.connected = True
        else:
            self.connected = False
            print("Scienta is not connected")
        return  

    def get(self,q=1):
        """ 
        Gets the current Scienta parameters
        returns the dictionary for q!=1
        """

        for key in vars(self):
            if key not in ["PHV","_Pcam","_savePlugin","_statsPlugin","wk","Frames","dtype"]:
                vars(self)[key]=caget(self.PHV+key+".VAL")
            if key  in ["LensMode","SpectraMode"]:
                vars(self)[key]=caget(self.PHV+key+".VAL",as_string=True)
            if key == "Frames":
                vars(self)[key]=caget(self.PHV.split(':')[0]+':'+"ExpFrames.VAL")
            if key == "wk":
                vars(self)[key]=caget(self.PHV+"WorkFunction")
        if q !=1:
            for key in vars(self):
                print(key+" = "+str(vars(self)[key]))
        return vars(self)

    def counts(self):
        """
        returns the current intensity on the Scienta using EA.
        """
        return caget(self._savePlugin+"Total_RBV")

    def _errorCheck(self,KineticEnergy,PassEnergy,LensMode,Frames,**kwargs):
        """
        check the parameters to see if they are allowed

        returns list of errors
        **kwargs:
            debug=False
        """
        kwargs.setdefault('debug',False)
        if kwargs['debug']:
            print("\n _errorCheck")
        error=[]
        
        if LensMode not in AllowedEnergyRange().keys():
            error.append("LensMode = "+LensMode+" is not a valid")
        elif PassEnergy not in AllowedEnergyRange()[LensMode]:
            error.append("PassEnergy = "+str(PassEnergy)+" is not a valid")
        try:
            KEmin,KEmax=AllowedEnergyRange(PassEnergy,LensMode)
            if KEmin>KineticEnergy or KineticEnergy>KEmax: 
                 error.append("Kinetic Energy: "+str(KineticEnergy)+" is outside allowed range for this Lens Mode,Pass Energy combination")
        except:
            error.append("Undefined PassEnergy/LensMode Combination "+str(PassEnergy
                                                                         )+"/"+LensMode)
        if Frames < 1:
            error.append("Frames must be 1 or greater")
        return error

    def put(self, KE=None, PE=None, LensMode="Angular",Frames=1,**kwargs):
        """
        Used to set all the SES parameters at once
        KE = kinetic energy (None keeps the current KE)
        PE = pass energy (None keeps the current PE)
        LensMode = "Angular"/"Transmission"
        
        **kwargs
            debug
        """
       
        kwargs.setdefault('debug',False)
        parms=self.get(q=1)
        if kwargs['debug']:
            print("\n EA.put")
            print('KE,PE,LensMode,Frames = ',KE,PE,LensMode,Frames)
        time.sleep(.1)
        if KE != None:
            parms.update({"KineticEnergy":KE})
        else:
            parms.update({"KineticEnergy":self.KineticEnergy})
        if PE != None:
            parms.update({"PassEnergy":PE})
        else:
            parms.update({"PassEnergy":self.PassEnergy})
        parms.update({"LensMode":LensMode})
        parms.update({"Frames":Frames})
       
        if kwargs['debug']:
            print("parms:",parms)
        
        time.sleep(.5)
        self._setHV(parms)
        
    def _setHV(self,parms,**kwargs):
        """
        Error checks and sets the Scienta high voltage supplies
            parms={'LensMode':LensMode,
               'PassEnergy':PassEnergy
               'KineticEnergy':KineticEnergy,
               'Frames':Frames}
               
        kwargs:
            q: quiet (default = 1); to print the dictionary q!=1
        """    
        kwargs.setdefault('q',1)
        kwargs.setdefault('debug',False)
        
        if kwargs['debug']:
            print('\n _setHV')
            print(parms)
            
        #checking for errors
        error=self._errorCheck(parms['KineticEnergy'],parms['PassEnergy'],parms['LensMode'],parms['Frames'])
        if kwargs['debug']:
            print(error)
        
        #setting Frames
        self._frames(parms['Frames'])
        
        if len(error) == 0:
            caput(self.PHV+"LensMode",parms["LensMode"])
            time.sleep(.25)  
            caput(self.PHV+"PassEnergy_Set",str(int(parms["PassEnergy"])))
            time.sleep(0.25)
            caput(self.PHV.split(':')[0]+':'+"ExpFrames",parms["Frames"])
            time.sleep(.25)
            caput(self.PHV+"KineticEnergy",parms["KineticEnergy"]+0.01)
            time.sleep(1)
            caput(self.PHV+"KineticEnergy",parms["KineticEnergy"])
            time.sleep(1)
    
            self.get()
            if kwargs['q'] !=1:
                for key in vars(self):print(key+" = "+str(vars(self)[key]))

        else:
            for e in error:
                #print(e) #JM need to modify
                x=5
        return        

    def off(self,quiet=True):
        """
        Zeros the high voltage supplies
        """
        self.zeroSupplies()
        if quiet == False:
            print ("EA HV is zeroed, visually confirm")

    def zeroSupplies(self):
        """
        Zeros the Scienta HV supplies (safe-state)
        """
        caput(self.PHV+"ZeroSupplies.RVAL",1)
        caput(self.PHV+"ZeroSuppliesSeq.PROC",1)


    def KE(self,val):
        """
        Sets the kinetic energy of the Scienta
        KE should be grearter than the PassEnergy 
        """
        self.put(KE=val)


    def _setwk(self,val):
        """
        Sets the wk to the value specified
        """
        caput(self.PHV+"WorkFunction",val)

    def PE(self,val):
        """
        Sets the pass energy of the Scienta
        PassEnergy: 1, 2, 5, 10, 20, 50, 100, 200, 500
        """
        self.put(PE=val)
        
    def lensMode(self,val):
        """
        Sets the pass energy of the Scienta
        LensMode: 'Transmission','Angular','Angular_01','Angular_05'
        """
        self.put(LensMode=val)

    def _frames(self,val,q=1):
        """
        Sets the number of frames
        Frames >=1
        """
        caput(self.PHV.split(':')[0]+':'+"ExpFrames.VAL",val)


    def _ElementSet(self,mode,q=1):
        """
        Used to switch between 
        0: High Energy (XPS) and 1:Low Energy (UPS) mode
        Must be done in conjunction with changing the cabling on the front of the HV supply
        mode = 0 or 1
        """
        caput(self.PHV+"ElementSet",mode)
        self.get()
        if q !=1:
            for key in vars(self):print(key+" = "+str(vars(self)[key]))
        return  

        
    def _BurstSupplies(self,val):
        """
        BurstSupplies:'Fast','Slow'
        """
        caput(self.PHV+"BurstSupplies",val,as_string=True)

    def _AcqCalc(self,val):
        """
        AcqCalc:'Off','Live','Scan'
        """
        caput(self.PHV+"AcqCalc",val,as_string=True)

    def _AcquisitionMode(self,val):
        """
        AcquisitionMode: 'Off','Scan','Spectra'
        """
        caput(self.PHV+"AcquisitionMode",val)

    def _ScientaMode(self,val):
        """
        ScientaMode: 'BetaScan','XYScan','Fixed','BabySweep','SweepOverlapping','SweepNo'
        """
        caput(self.PHV+"ScientaMode",val,as_string=True)
    
    
    def _babySweepSteps(self,val):
        """
        sets the number of steps in baby sweep
        """
        caput(self.PHV+"babySweepSteps.VAL",val)

    def _spectraStatus(self):
        """
        checking the spectra status
        returns 1=busy or 0=idle
        """
        status=caget(self.PHV+'ScanTrigger')
        return status

    def _spectraProgress(self):
        """
        Monitors the spectra status every 30 seconds
        returns 0 when the spectra scan is idle
        """
        while True:
            status=self._spectraStatus()
            if (status==1):
                time.sleep(30)
            else:
                break
        return 0
    
    def _SpectraMode(self,Mode):
        """
        Checks that Mode exists and sets the mode 

        """
        DefinedModes=["Fixed","Baby-Sweep","Sweep"]
        if Mode in DefinedModes:
            caput(self.PHV+"SpectraMode", "Mode")
        else: 
            print("Not a defined SpectraMode")

    def _spectraInfo(self):
        """
        returns the pertainent information for a given spectra type
            modeNum=2; Swept=[start,stop,step]
        """
        modeNum=caget(self.PHV+"SpectraMode")
        scanType=caget(self.PHV+"SpectraMode",as_string=True)
        info=[]
        if modeNum==2: #swept
            info.append(caget(self.PHV+"sweepStartEnergy"))
            info.append(caget(self.PHV+"sweepStopEnergy"))
            info.append(caget(self.PHV+"sweepStepEnergy"))
        elif modeNum==1: #baby-swept
            info.append(caget(self.PHV+"babySweepCenter.VAL"))
        elif modeNum==0: #fixed
            info.append(caget(self.PHV+"fixedEnergy.VAL"))
        return scanType,info

    def live(self,KE=None,PE=None,LensMode="Angular"):
        """
        puts the Scienta back in live mode (no savinga and Fixed Mode) with PE and KE
            if PE or KE is None then uses the current value
        """
        if self.spectraProgress() != 1:
            parms=self.get()
            caput('29idcScienta:HV:LiveSeq',1)
            self.put(KE,PE,LensMode)
        else:
            print("Scienta spectra in progress")
            
    def _updateAttributes(self,filepath=None):
        """
        filepath is the full path to the xml file
            filepath = None; to update after making changes
            filepath='/xorApps/epics/synApps_6_1/ioc/29idcScienta/iocBoot/ioc29idcScienta/HDF5Attributes.xml'

        from 29idcScienta:HDF1: screen
            More (under process plugin)
                NDPluginBase Full
        """
        if filepath is None:
            print(caget(self._savePlugin+"NDAttributesFile",as_string=True))
        else:
            caput(self._savePlugin+"NDAttributesFile",filepath)
            time.sleep(1)
            print(caget(self._savePlugin+"NDAttributesStatus",as_string=True))

        
    def _spectraErrorCheck(self,KElist,PassEnergy,Frames,LensMode,**kwargs):
        """
        error checking to see if parameters are allowed
        """
        error=[]
        parms={}
        for KE in list(KElist[0:-1]):
            error=self._errorCheck(KE,PassEnergy,LensMode,Frames,**kwargs)   

        if len(error) > 0:
            print(error)
        return error
    
    def _spectra_Swept(self,KEstart,KEstop,KEstep,PE):
        """
        setting up for a swept spectra, does not set the LensMode, PassEnergy or Frames
        """
        print("setting SpectraMode")
        caput(self.PHV+"SpectraMode", "Sweep")
        time.sleep(1)
        #input pass energy setpoint (doesn't change util KE is changed but needed for calcs)
        caput(self.PHV+"PassEnergy_Set",PE)
        time.sleep(0.25)
        #input spectra parameters
        caput(self.PHV+"sweepStartEnergy.VAL",KEstart);time.sleep(0.25) #added 20220317 - sweep calcs need time to process or you get weird stuff
        caput(self.PHV+"sweepStopEnergy.VAL",KEstop);time.sleep(0.25) #added 20220317 - sweep calcs need time to process or you get weird stuff
        caput(self.PHV+"sweepStepEnergy.VAL",KEstep)
        time.sleep(1)
        #Re-Calc
        #print('Yes Re-Calc')
        #caput(self.PHV+"SweepReCalc.PROC",1)
        #time.sleep(1)
        #Load spectra parameters
        caput(self.PHV+"SweepLoadFanout.PROC",1)
        time.sleep(1)


    def _spectra_BabySwept(self, KEcenter):
        """
        setting up for a Baby-Swept
        """
        #setting SpectraMode
        caput(self.PHV+"SpectraMode", "Baby-Sweep")
        time.sleep(1)
        #input spectra parameters
        caput(self.PHV+"babySweepCenter.VAL",KEcenter)
        time.sleep(1)
        #Load spectra parameters
        caput(self.PHV+"SweepLoadFanout.PROC",1)
        time.sleep(1)
        #settling time
        #caput(self.PHV+"scan2.PDLY",0.25)
        #caput(self.PHV+"scan2.DDLY",0.1)

    def _spectra_Fixed(self,KEcenter):
        """
        setting up for Fixed
        """
        #setting SpectraMode
        caput(self.PHV+"SpectraMode", "Fixed")
        caput("29idcScienta:HV:SpectraMode", "Fixed")
        time.sleep(1)
        #input spectra parameters
        caput(self.PHV+"fixedEnergy.VAL",KEcenter)
        time.sleep(1)
    
    def _spectraSetup(self,EAlist,**kwargs):
        """ 
        Writes EAlist to the appropriate PVs and does error checking
        and return scanType, scanPV, KElist, parms
        **kwargs
            run = True; will write the PVs (False just does error checking)
            LensMode = "Angular"
            debug = False
    
        """
        kwargs.setdefault("debug",False)
        kwargs.setdefault("run",True)
        kwargs.setdefault("LensMode","Angular")
        
        if kwargs['debug']:
            print("\n\n _spectraSetup")
            print("EAlist",EAlist)
            
        scanType="spectra"
        if EAlist[-1] == "BS":
            BS=True
            EAlist=EAlist[0:-1]
        else: 
            BS=False
        Sweeps=EAlist[-1]
        Frames=EAlist[-2]
        PassEnergy=EAlist[-3]
        EnergyMode=EAlist[0]
        KElist=np.array(EAlist[1:-3])
        LensMode=kwargs['LensMode']


        if kwargs['debug']:
            print("KElist: ",KElist)
        
        parms={'LensMode':LensMode,'PassEnergy':PassEnergy,'KineticEnergy':KElist[0],'Frames':Frames}
                
        #checking parameters are valid
        error=self._spectraErrorCheck(KElist,PassEnergy,Frames,LensMode)
        if len(error) == 0: #passed check
            #Checking Progress
            if self._spectraStatus() == 1:
                print("Spectra in Progess")
            
            # writing scan parameters
            else:
                if len(EAlist)==7: #Swept Mode
                    scanType="Sweep"
                    self._spectra_Swept(KElist[0],KElist[1],KElist[2],PassEnergy) 
                    parms={'LensMode':LensMode,'PassEnergy':PassEnergy,'KineticEnergy':KElist[0]-.01,'Frames':Frames}
                elif len(EAlist)==5: #fixed or baby sweep
                    if BS is True:
                        scanType="Baby-Sweep"
                        self._spectra_BabySwept(KElist[0])
                        parms={'LensMode':LensMode,'PassEnergy':PassEnergy,'KineticEnergy':KElist[0]-.01,'Frames':Frames}
                    else:
                        scanType="Fixed"
                        self._spectra_Fixed(KElist[0])
                        parms={'LensMode':LensMode,'PassEnergy':PassEnergy,'KineticEnergy':KElist[0]-.01,'Frames':Frames}
                else:
                    print("not a valid number of parameters")
                #PV to initate scan"
            scanPV=self.PHV+"ScanTrigger"
            return scanType, scanPV, KElist, parms

    def _spectraMessage(self,scanType, scanPV, KElist):
        """
        prints KE range for spectra
        """
        message=str(scanType)+" scan  KE: "
        for KE in KElist:
            message+=str(KE)+" | "
        return message[0:-2]
    
       

    def spectra(self,EAlist,**kwargs):
        """
        takes a Scienta spectra based on the number or parameters in EAlist
            Sweeps are handled by scanRecord outside of this modual (see scanEA)
        
        Fixed Mode:["KE",CenterEnergy,PassEnergy,Frames,Sweeps] (5)
        Swept Mode:["KE",StartEnergy,StopEnergy,StepEnergy,PassEnergy,Frames,Sweeps] (7)
        Baby Sweep (dither):["KE",CenterEnergy,PassEnergy,Frames,Sweeps,"BS"] (6)
            
        **kwargs:
            debug=False
            run=True (executes the scan); False for error checking only
            LensMode = Angular (default)
        """
        kwargs.setdefault("debug",False)
        kwargs.setdefault("run",True)
        
        #Sending Parmeters to _spectraSetup 
        scanType, scanPV, KElist, parms =self._spectraSetup(EAlist,**kwargs)
        
        if kwargs['debug']:
            print(scanType,scanPV, KElist)
        else:
            print(scanType, "KE: ", KElist)
        #Getting Filename info
        fpath=caget(self._savePlugin+"FileName_RBV",as_string=True)+"_ "
        fnum=caget(self._savePlugin+"FileNumber",as_string=True)
       
        
        if kwargs['run'] is True:
            dateandtime=time.strftime("%a %d %b %Y %H:%M:%S",time.localtime())
            print("\t"+fpath+fnum+" started at ",dateandtime,"\r")
            self.put(parms['KineticEnergy'],parms['PassEnergy'],LensMode=parms['LensMode'],Frames=parms['Frames'])
            caput(scanPV,1,wait=True,timeout=129600)#timeout=36 hours
            dateandtime=time.strftime("%a %d %b %Y %H:%M:%S",time.localtime())
            print("\t"+fpath+fnum+" finished at ",dateandtime,"\r")


