from numpy import nan

from epics import caput, caget
#from iexcode.instruments.IEX_endstations import *

##############################################################################################################
################################            default positions             ##############################
##############################################################################################################

###### We should make pvs in the 29idb ioc to hold these values:
def diagnostics_dict():
    """
    Dictionary of Diagnostic positions In and Out by either motor number or name
    WARNING: When updating motor values, also update the following screens:
        - 29id_BL_Layout.ui               (for MeshD and DiodeC)
        - 29id_Diagnostic.ui
        - 29idd_graphic
    usage:       
        diagnostics_dict()['name'] returns dictionary motor:name
        diagnostics_dict()['motor'] returns dictionary name:motor   
        diagnostics_dict()['In'] returns dictionary motor:In position  (where val can be a list for multiple position)  
        diagnostics_dict()['Out'] returns dictionary motor:In position  
                motor=diagnostics_dict()['motor']['gas-cell']
                pos_in=diagnostics_dict()['In'][motor]
                
    WARNING: When updating MeshD (D5D) value: update value in the dictionnary + caQtdM (29id_BL_Diag.ui + 29idd_graphic.ui + Diagnostic.ui)
    
    Previously: AllDiag_dict
    """
    diag={}
    diag["In"]  = {                5:-55, 6:-46,          17:-56, 20:-30, 25:-56, 28:[-3,-3]}
                                                                                    
    diag["Out"] = {1:-4, 2:-10, 3:-4, 4:-4, 5:-20, 6:-20, 7:-20, 17:-20, 20:-21, 25:-20, 28:-3}    
    diag["name"]= {1:"H-wire", 2:"V-wire", 3:"H-Diagon", 4:"V-Diagon", 5:"W-mesh",
     6:"D2B", 7:"D3B", 17:"D4C/pre-slit", 20:"gas-cell", 25:"D4D/pre-slit", 28:"D5D/pre-RSXS"}
    diag["motor"]= {"H-wire":1, "V-wire":2, "H-Diagon":3, "V-Diagon":4,"W-mesh":5,
     "D2B":6, "D3B":7, "D4C":17, "gas-cell":20,"D4D":25,"D5D":28}
    return diag

def diagnostic_CA_dict():
    CA_dict={
        'diode_c':'29idb:ca15:'
    }
    return CA_dict


def diagnostic(diagnostic_name,in_out):
    "Inserts/retracts a diagnostic(motor number or name) either = \"In\" or \"Out\""
    diag=diagnostics_dict()
    if type(diagnostic_name) is int:
        motor=diagnostic_name
        name=diag['name'][motor]
    else:
        name=diagnostic_name
        motor=diag["motor"][name]
    position=diag[in_out][motor]

    caput("29idb:m"+str(motor)+".VAL",position,wait=True,timeout=18000)
    print("\n"+name+": "+ in_out)

def diagnostics_all_out(diode_stay_in=False):
    """
    Retracts all diagnostic
    diode_to_stay_in = True / False (checks beamline)

    Previously: AllDiagOut
    """
    diag=diagnostics_dict()
    text=""

    #which motor is Diode of interest
    if diode_stay_in:
        branch = BL.branch

    if branch == 'c':
        diode_motor=diag["motor"]["gas-cell"]
    elif branch == 'd':
        diode_motor=diag["motor"]["D5D"]
    else:
        diode_motor=None

    #Taking out the diagnostic
    for motor in list(diag["Out"].keys()):
        if motor is diode_motor:
            text=' except Diode-'+BL.branch
            #putting Diode In if not already in -JM
            position=diag["In"][motor]
            caput("29idb:m"+str(motor)+".VAL",position,wait=True,timeout=18000)
        else:
            position=diag["Out"][motor]
            caput("29idb:m"+str(motor)+".VAL",position,wait=True,timeout=18000)
    text="All diagnostics out"+text
    print("\n",text)

def diagnostics_all_in():
    """
    Inserts all diagnostic (meshes and diodes) for pinhole scans
    
    Previously: AllDiagIn and AllMeshIn()
    """
    diag=diagnostics_dict()
    for motor in list(diag["In"].keys()):
        position=diag["In"][motor]
        if type(position) == list:
            position=position[0]
        caput("29idb:m"+str(motor)+".VAL",position,wait=True,timeout=18000)
        print('m'+str(motor)+' = '+str(position))
    print("All diagnostics in (meshes and diodes) for pinhole scans")

def mesh_W(In_Out):
    """
    Inserts/retracts RSXS mesh (post-slit); arg = \"In\" or \"Out\"

    Previously MeshW
    """
    diag=diagnostics_dict()
    motor=5; position=diag[In_Out][motor]
    caput("29idb:m"+str(motor)+".VAL",position,wait=True,timeout=18000)
    print("\nD1A mesh_W: "+ In_Out)


def diodeC(In_Out):
    """
    Inserts/retracts ARPES (gas-cell) diode; arg = \"In\" or \"Out\"
    
    Previously: DiodeC
    """
    diag=diagnostics_dict()
    motor=20; position=diag[In_Out][motor]
    caput("29idb:m"+str(motor)+".VAL",position,wait=True,timeout=18000)
    print("\nARPES Diode: "+ In_Out)


def diodeD(In_Out):
    """
    Inserts/retracts RSXS diode; arg = \"In\" or \"Out\"

    Previously:DiodeD
    """
    diag=diagnostics_dict()
    motor=28; position=position=diag[In_Out][motor]
    if type(position) == list:
        position=position[1]
    caput("29idb:m"+str(motor)+".VAL",position,wait=True,timeout=18000)
    print("\nRSXS Diode: "+ In_Out)



def meshC(In_Out,which="postSlit"):
    """
    Inserts/retracts ARPES mesh (preSlit or postSlit); arg = \"In\" or \"Out\"
    
    Previously: MeshC
    """
    diag=diagnostics_dict()
    if which == "postSlit":
        motor=20; position=-31
    elif which == "preSlit":
        motor=17; position=diag[In_Out][motor]

    caput("29idb:m"+str(motor)+".VAL",position,wait=True,timeout=18000)
    print("\nD4C Au-Mesh: "+ In_Out)

def meshD(In_Out):
    """
    Inserts/retracts RSXS mesh (post-slit); arg = \"In\" or \"Out\"
    
    Previoulsy: MeshD
    """
    diag=diagnostics_dict()
    motor=28; position=position=diag[In_Out][motor]
    if type(position) == list:
        position=position[0]
    caput("29idb:m"+str(motor)+".VAL",position,wait=True,timeout=18000)
    print("\nD5D Au-Mesh: "+ In_Out)

def diagnostic_read(diode_d, quiet=True):
    """
    reads the current amplifier and returns the value
    quiet = False to pring
    """ 
    try:
        diode_pv = diagnostic_CA_dict['diode_d']
        val=caget(diode_pv+'read')
    except:
        diode_pv="not connected"
        val = nan
        print('pv not defined')

    if not quiet:
        print('Diode-D ',val, "(pv = "+diode_pv+")")
    return val

def diodeC_read(quiet=True):
    """
    reads the current amplifier and returns the value
    quiet = False to pring
    """ 
    val = diagnostic_read('diode_d',quiet)
    return val

def diodeD_read(quiet=True):
    """
    reads the current amplifier and returns the value
    quiet = False to pring
    """ 
    val = diagnostic_read('diode_d',quiet)
    return val

def meshD_read(quiet=True):
    """
    reads the current amplifier and returns the value
    quiet = False to pring
    """ 
    val = diagnostic_read('diode_d',quiet)
    return val


###############################################################################################
####################################### FLUX CONVERSION #######################################
###############################################################################################




