from os.path import join,isfile

from epics import caget, caput

from iexcode.instruments.utilities import today

## To do: make this work if User Folder doesn't already exist

##############################################################################################################
##############################            general   logfile functions          ##############################
##############################################################################################################
class Logfile:
    def __init__(self,endstation_name,user_name,headerlist,entries_function):
        """
        endstation: used for filename pv ('ARPES' / 'Kappa' / 'Octupole' / 'Test') 
        user_name: used for filepath
        headerlist: header info

        """
        self.endstation_name = endstation_name
        self.pv = self._name_pv()
        self.userName = user_name
        self.filepath = self._fpath()
        self.headerlist = headerlist
        self.entries_function = entries_function

        self.name_set()

    def _name_pv(self):
        """
        Dictionary to get the PV in which the FileName for logging is stored
        
        endstation: 'ARPES' / 'Kappa' / 'Test

        Previously: logname_pv
        """
        pv_dict={
            'Test':'29idb:userStringSeq10.STR1',
            'ARPES':'29idb:userStringSeq10.STR2',
            'Kappa':'29idb:userStringSeq10.STR3',
            'Octupole':'29idb:userStringSeq10.STR4',
            }
        if self.endstation_name in pv_dict.keys():
            pv=pv_dict[self.endstation_name]
        else:
            pv='29idb:userStringSeq10.STR10'
        return pv

    def name_set(self,**kwargs):
        """
        Sets the string used for the FileName in scanlog and EAlog
        uses logname_PV to reference PV associated with a particular ioc
        **kwargs:
            filename = to set a specific filename, default => YYYYMMDD_log.txt

        Previously: logname_set
        """
        kwargs.setdefault('filename_suffix','_log')
        kwargs.setdefault('filename',today()+kwargs['filename_suffix']+'.txt')

        filename = kwargs['filename']

        try:
            caput(self._name_pv(),filename)
            print("\nLog filename = \'"+filename+"\' @ "+self._name_pv())
            print('To change filename, use logname_set(endstation_name,new_name)')
        except:
            print("Error: was not able to set the filename, check that 29idb ioc is running")

        
    def name_get(self):
        """
        Gets the string used for the FileName in scanlog and EAlog for the given scanIOC.

        Previously: logname
        """

        return caget(self._name_pv())
       
    def _fpath(self):
        """
        returns the full path to the logfile based on the current user in the mda scanRecord


        Previously: logname, logname_generate
        """
        try: 
            filename = self.name_get()
            user_name = self.userName
            fpath_with_subfolder = join(user_name,filename)
        except:
            fpath_with_subfolder = "logfile.txt"
            self.name_get()
            print("Couldn't read log file path, using: "+fpath_with_subfolder)
            
        return fpath_with_subfolder


    def print(self, comment=''):
        """
        Writes comments to the logfile on a new line
        comment is a string of what you want to add to the file 
            e.g. comment = "------- New Sample--------"
        
        Previously: logprint
        """

        try:
            if not isfile(self.filepath):
                self.write_header()
            with open(self.filepath,'a') as myfile:
                myfile.write(comment)
                myfile.write('\n')

        except:
            print("Logprint failed")


    def write_header(self): 
        """
        file path = path to where file_name lives
        file_name = name of the logfile
        entry is a list

        Previously: SaveFile_Header
        """  
        version = '1.4'
        
        with open(self.filepath, "w+") as f:
            f.write('@Log version: '+ version+'\n\n')
            f.write('FilePath '+self.endstation_name+': '+self.filepath+'\n')

            for key in self.headerlist.keys():
                f.write(key+' Header:  '+ self.headerlist[key]+'\n\n')


   
    def update(self,entry_list,pv_list,format_list):
        """ 
        To be used for scanlog and scanEA functions.
        Update SaveFile_Header version number when changing the structure of the file (indexing).

        Previously: SaveFile
        """
        #entry_list is the human readable portion
        
        if not isfile(self.filepath):
            self.write_header()

        with open(self.filepath, "a+") as f:
            for i in range(len(format_list)):
                pv_format = "{0:"+format_list[i]+"},"
                f.write(pv_format.format(pv_list[i]))
            last_entry=len(format_list)-1
            pv_format="{0:"+format_list[last_entry]+"}\n"
            f.write(pv_format.format(pv_list[last_entry]))


