from time import sleep

from epics import caget, caput

import iexcode.instruments.cfg as iex
from iexcode.instruments.utilities import read_dict, print_warning_message

M0M1_fpath="/home/beams22/29IDUSER/Documents/User_Macros/Macros_29id/IEX_Dictionaries/Dict_IDCal.txt"

def _FMB_mirror_ioc(mirror_num):
    """
    returns the ioc name for the given mirror number: 
    miror_num = 0 / 1 / 3  for M0 / M1 /M3R respectively
    """
    ioc = '29id_'+['M0','M1','','M3R'][mirror_num]+":"
    return ioc

def FMB_mirror_status(mirror_num):
    """
    returns the ioc name for the given mirror number: 
    miror_num = 0 / 1 / 3  for M0 / M1 /M3R respectively

    status =1 when positioned
    """
    ioc = '29id_'+['M0','M1','','M3R'][mirror_num]+":"
    pv = _FMB_mirror_ioc(mirror_num)
    status=caget(pv+'SYSTEM_STS')

    return status

def FMB_mirror_axis_position(mirror_num,axis, verbose=True):
    """
    returns the readback and set point of an FMB mirror
    """
    pv = _FMB_mirror_ioc(mirror_num)
  
    rbv = round(caget(pv+axis+"_MON"),3)
    sp = round(caget(pv+axis+"_POS_SP"),3)

    if verbose:
        print (pv+axis+": "+str(rbv))
    
    return rbv,sp

def FMB_mirror_get(mirror_num,verbose=True):
    """
    get and returns the current mirror position 
    """
    axis_labels=['TX','TY','TX','RX','RY','RZ']
    vals=[]
    message = "\nM"+str(mirror_num)+" @ "
    for axis in axis_labels:
        rbv,sp = FMB_mirror_axis_position(mirror_num,axis, verbose=False)
        vals.append(rbv)
        message =+ "%.3f"+"/" % rbv 
    if verbose:
        print(message)
    return vals

def FMB_mirror_move(mirror_num,axis,val,verbose=True):
    """
    miror_num = 0 / 1 / 3  for M0 / M1 /M3R respectively
    and axis:
        TX = lateral                 RX = Roll
        TY = vertical                RY = Pitch
        TZ = longitudinal            RZ = Yaw

    "Previously: Move_M0M1
    """  
    pv = _FMB_mirror_ioc(mirror_num)
    axes = ['TX','TY','TZ','RX','RY','RZ']
    if axis in axes:
        caput(pv+axis+"_SP.PREC",3)  
        caput(pv+axis+"_POS_SP")
        caput(pv+"MOVE_CMD.PROC",1,wait=True,timeout=18000)
        while True:
            if FMB_mirror_status != 1:
                sleep(.5)
            else:
                break
        FMB_mirror_axis_position(mirror_num,axis,verbose=verbose)
            
    else:
        print_warning_message(axis+' is not a valid axis chose '+axes)

def FMB_mirror_move_all(mirror_num,position_list,verbose=True):
    """
    miror_num = 0 / 1 / 3  for M0 / M1 /M3R respectively
    position_list = [TX,TY,TZ,RX,RY,RZ]

    "Previously: Move_M0M1
    """  
    pv = _FMB_mirror_ioc(mirror_num)
    for axis in ['TX','TY','TZ','RX','RY','RZ']:
        caput(pv+axis+"_SP.PREC",3)  
        caput(pv+axis+"_POS_SP")
    caput(pv+"MOVE_CMD.PROC",1,wait=True,timeout=18000)
    while True:
        if FMB_mirror_status != 1:
            sleep(.5)
        else:
            break
    FMB_mirror_axis_position(mirror_num,axis,verbose=verbose)
    

def FMB_mirror_tweak(mirror_num,axis,val,verbose=False):
    """
    miror_num = 0 / 1 / 3  for M0 / M1 /M3R respectively
    and axis:
        TX = lateral                 RX = Roll
        TY = vertical                RY = Pitch
        TZ = longitudinal            RZ = Yaw

    "Previously: Move_M0M1
    """
    pv = _FMB_mirror_ioc(mirror_num)
    previous_position = FMB_mirror_axis_position(mirror_num,axis,verbose=False)
    new_position = previous_position[0] + val
    FMB_mirror_move(mirror_num,axis,new_position,verbose=False)

    if verbose:
     print(pv+" "+str(previous_position[0])+" -> "+str(new_position[0]))


def FMB_mirror_scan(mirror_num,start,stop,step):
    """
    e.g. Scan_FMB_mirror("m1:TX",-5,5,.25)

        TX = lateral                 RX = Yaw
        TY = vertical                RY = Pitch
        TZ = longitudinal            RZ = Roll
    """
    pv = _FMB_mirror_ioc(mirror_num)
    
    # database sets .PREC==0.  We want more digits than that.
    caput(pv+"_SP.PREC",3)
    iex.BL.mda.fillin(pv+"_SP",pv+"_MON",start,stop,step)




def M0M1_table(run,mirror):
    """
    Prints the positions TX / TY / TZ / RX / RY / RZ for either Mirror = 0 or 1 (M0 or M1) for the specified Run
    Run='default' give a reasonable starting position after homing
    M0M1_SP() will put those values as the set points, you will need to push the Move button
    """
    M0M1_Pos=read_dict(M0M1_fpath)
    return M0M1_Pos[run][mirror[-1]]

def M0M1_SP(run,mirror,Go=False):
    """
    Gets the values from the mirror position from a previous run and
    put values as defined by M0M1_Table as the set points
    Go = True / False (default); True to moves to that position
    """
    mirror_pos=M0M1_table(run,mirror).split('/')
    motor=['TX','TY','TZ','RX','RY','RZ']
    mirror=mirror[-1]
    for i in range(len(motor)):
        PV="29id_m"+str(mirror)+":"+motor[i]+"_POS_SP"
        Val=mirror_pos[i] #float(MirrorPos[i])
        print(PV+" = "+Val)
        caput(PV,Val)
    sleep(0.5)
    if Go:
        caput('29id_m'+str(mirror)+':MOVE_CMD.PROC',0,wait=True,timeout=18000)
    else:
        print(" caput(\'29id_m"+str(mirror)+":MOVE_CMD.PROC\',0)")
