from iexcode.instruments.scanRecord import ScanRecord
from iexcode.instruments.utilities import print_warning_message

import iexcode.instruments.cfg as iex

"""
Beamline_Config class is used to contain information about ioc, branch, mode

it makes the default stuff work

this will prompt for the endstation and will set the default parameters, look at the 
init kwargs to see which defaults you can change.
"""
#For new endstation modify here:
endstations_list = ['ARPES','Kappa']   

class Beamline_Config:
    """
    used for defining which endstation in which you are running
    for short names and ioc info

    BL = Endstation('ARPES')
        BL.endstation_name: name of endstation  ("ARPES" / "Kappa" / "Octupole")
        
        BL.ioc: string of scan ioc  ('29idb:', '29idTest:')
        BL.mda: scanRecord object of ioc specified by .ioc 
        
        BL.xrays: True/False, used for detectors and shutter check
        BL.mode: 'user' / 'staff', used for detectors and to specifiy folder
        
        BL.folder: shorthand for where data is save and used for saveData structure ('b','c','d')
        BL.prefix: mda prefix (set by endstation)

        BL.log: object of log class which holds logging info

    """

    def __init__(self,endstation_name,scan_ioc,xrays,mode='user',Motors=None,**kwargs):
        """
        intializes the several beamline variables 

        endstation_name = 'ARPES' / 'Kappa'
        
        BL = Endstation()
            BL.endstation_name => endstation_name
            BL.branch => BL branch 'c' or 'd' only (used for shutter checks)
            BL.xrays => True/False 
            BL.mode => 'user'/'staff' previously: BL_Mode_Set
            BL.mda => scanRecord class used for scanning
            BL.folder => 'b','c','d'
            BL.prefix => 'ARPES_','Kappa_'
            BL.ioc => previously: BL_ioc()
            BL.Motors => motor calls
            BL.safe_state => function to put endstation in a 'safe state'
            BL.endstation_get => function to return endstation status
            BL.log => log class

            Note intialization does not set default detectors and logging for .mda
            
        """
       
        self.endstation_name = None
        self.branch = None
        self.ioc = scan_ioc
        self.folder = None
        self.prefix = None

        self.mode = None
        self.xrays = xrays

        self.mda = ScanRecord(scan_ioc)
        self.motors = Motors

        self.safe_state = None
        self.endstation_get = None
        
        BL_mode_list = ['user','staff']

        if endstation_name in endstations_list:
            self.endstation_name=endstation_name
        else:
            print('Not a valid Endstation choice')
            print('Endstations: '+endstations_list)
            return

        
        if mode in BL_mode_list:
            self.mode = mode
            if mode == 'user':
                #For new endstation modify here:
                if endstation_name == 'ARPES':
                    self.folder = 'c'
                    self.prefix = 'ARPES_'
                    self.branch = 'c'
                elif endstation_name == 'Kappa':
                    self.folder = 'd'
                    self.prefix = 'Kappa_'
                    self.branch = 'd'
                #elif endstation_name == 'Octupole':
                    #self.folder = 'e'
                    self.branch = 'd'
                else:
                    print('folder and prefix not set')

            elif mode == 'staff':
                #overwrite folder for staff mode
                self.folder = 'b' 
                self.prefix = endstation_name+"_"
        else:
            message = 'Not a valid BL_mode choice\n'
            message += 'BL_modes: '+BL_mode_list
            print_warning_message(message)
            return  

        if 'safe_state' in kwargs:
            self.safe_state = kwargs['safe_state']
        if 'endstation_get' in kwargs:
            self.endstation_get = kwargs['endstation_get']



##############################################################################################################
##############################               BL commands            ##############################
##############################################################################################################
def BL_ioc():
    """
    returns the ioc from the Endstation instance
    """
    return iex.BL.ioc

def BL_branch():
    """
    returns the branch from the Endstation instance
    """
    return iex.BL.branch

def BL_mode():
    """
    returns the beamline mode, User / Staff / No_Xray
    """
    return iex.BL.mode

def BL_mda_prefix():
    """
    returns the mda file prefix
    """
    return iex.BL.prefix

def BL_mda_filepath():
    """
    returns the mda file prefix
    """
    return iex.BL.filepath

##############################################################################################################
def get_all(verbose=True):
    """
    returns the current beamline and endstation setting

    """
    vals.update(iex.BL.endstation_get(verbose))
    
    return vals
