from numpy import inf,nan

from epics import caget, caput
from .IEX_BL_config import *


from iexcode.instruments.shutters import main_shutter_close
from iexcode.instruments.utilities import print_warning_message
from iexcode.instruments.encoders import encoders_reset_zero

slit_ioc="29idb:"

def slits_dictionary():
    """
    dictionary of slit pv names for two and four blade slit assemblies

    slit_name = slit1A, slit2B, slit3D
    """
    #'slit3C' : (None, "uses a flexure stage so is different"),
    
    d = {
        'slit1A' : ('Slit1H','Slit1V'),
        'slit2B' : ('Slit2H','Slit2V'),
        'slit3D' : ('Slit4V')
        }
    return d

def slits_pvs(slit_name):
    """
    returns the rbv and drive for the size,center
    (rbv_size,val_size),(rvb_center,val_center)
    """
    slit_pv = slit_ioc + slit_name
    size_rbv = slit_pv +'t2.D'
    size_val = slit_pv +'center.VAL'
    center_rbv = slit_pv +'t2.C'
    center_val = slit_pv +'center.VAL'

    return (size_rbv,size_val),(center_rbv,center_val)

def slits_synch(slit_name):
    """
    synch the motor position and the slit table for all the beamline slits
    """
    slit_pv = slit_ioc + slit_name+'sync.PROC'
    caput(slit_pv,1)

def slits_sync_all():
    """
    synch the motor position and the slit table for all the beamline slits
    Slit-1A, slit-2B, slit-3C, slit-3D

    Previously: SyncAllSlits
    """
    slit_list = slits_dictionary()
    for slit_name in slit_list.keys():
        slits_synch(slit_name)

def slits_print_all():
    """
    gets the current position of slit-1A, slit-2B, slit-3C, slit-3D

    Previously Get_Slits
    """
    slits_sync_all()
    slit1A_get(verbose=True)
    slit2B_get(verbose=True)

    slit3C_get(verbose=True)
    slit3D_get(verbose=True)

def slits_get_all(verbose=False):
    """
    returns a dictionary with the current slit status
    """
    vals = {}
    vals['slit1A_size'], vals['slit1A_center'] = slit1A_get(verbose=False)
    vals['slit2B_size'], vals['slit2B_center'] = slit2B_get(verbose=False)
    vals['slit3C_size'] = slit3C_get(verbose=False)
    vals['slit3D_size'],vals['slit3D_center'] = slit3D_get(verbose=False)

    if verbose:
        slits_print_all()
    return vals

def slits_set(slit_name,size,center,verbose=True):
    """
    synchs the slit motors and the slit table and then sets the center and size
    
    slit_name is key in slits_dictionary
    size = (H_size, V_size) 
    center = (H_center, V_center)
    
    Previously: SetSlit
    """
    d = slits_dictionary()
    #syncing
    slits_synch(slit_name)

    for i,slit_HV in enumerate(d[slit_name]): #H and V
        (size_rbv,size_val),(center_rbv,center_val) = slits_pvs(slit_HV)
        caput(size_val, size[i],timeout=180)
        caput(center_val, center[i],timeout=180)

    if verbose:
        if (d[slit_name])>1:
            print(slit_name + " = ("+str(round(size[0],3))+"x"+str(round(size[1],3))+") @ ("+str(center[0])+","+str(center[1])+")")
        else:
            print(slit_name + " = ("+str(round(size[0],3))+") @ ("+str(center[0])+")")

    return size,center


def slits_get(slit_name,verbose=True):
    """
    returns the current H_size,V_size,H_center,V_center
    
    """
    d=slits_dictionary()
    size = []
    center = []

    #syncing
    slits_synch(slit_name)

    for slit_HV in d[slit_name]: #H and V
        (size_rbv,size_val),(center_rbv,center_val) = slits_pvs(slit_HV)
        size.append(caget(size_rbv))
        center.append(caget(center_rbv))

    if verbose:
        if (d[slit_name])>1:
            print(slit_name + " = ("+str(round(size[0],3))+"x"+str(round(size[1],3))+") @ ("+str(center[0])+","+str(center[1])+")")
        else:
            print(slit_name + " = ("+str(round(size[0],3))+") @ ("+str(center[0])+")")

    return tuple(size),tuple(center)

def slits_set_size(slit_name,size):
    """
    sets the slit size with the current slit center

    size = (H,V) for slit1A, slit2B
    size = V for slit3D
    """
    size_current,center = slits_get(slit_name,verbose=False)
    slits_set(size,center)

def slits_set_center(slit_name,center):
    """
    sets the slit size with the current slit center

    center = (H,V) for slit1A, slit2B
    center = V for slit3D
    """
    size,center_current = slits_get(slit_name,verbose=False)
    slits_set(size,center)

def slits_scan_size(slit_name,direction,start,stop,step,**kwargs):
    """
    slit_name = 'slit1A','slit2B','slit3D'
    direction = 'H', 'V'
    **kwargs
        center: slits center,  if not specified then uses current center position
    """
    kwargs.setdefault('center',slits_get(slit_name,verbose=False)[1])
    kwargs.setdefault('execute',True) 

    slit_H,slit_V = slits_dictionary()[slit_name]
       
    if direction == 'H':
        (size_rbv,size_val),(center_rbv,center_val) = slits_pvs(slit_H)
    if direction == 'V':
        (size_rbv,size_val),(center_rbv,center_val) = slits_pvs(slit_V)

    #set slit center
    slits_set_center(slit_name,kwargs['center'])
    #scan
    BL.mda.fillin(size_rbv,size_val,start,stop,step,**kwargs)
    if kwargs['execute']:
        BL.mda.go(**kwargs)

def slits_scan_center(slit_name,direction,start,stop,step,**kwargs):
    """
    slit_name = 'slit1A','slit2B','slit3D'
    direction = 'H', 'V'
    **kwargs
        size: slits size, if not specified then uses current size
    """
    kwargs.setdefault('size',slits_get(slit_name,verbose=False)[0])
    kwargs.setdefault('execute',True) 

    slit_H,slit_V = slits_dictionary()[slit_name]

    if direction == 'H':
        (size_rbv,size_val),(center_rbv,center_val) = slits_pvs(slit_H)
    if direction == 'V':
        (size_rbv,size_val),(center_rbv,center_val) = slits_pvs(slit_V)
    
    #set slit center
    slits_set_size(slit_name,kwargs['size'])
    #scan
    BL.mda.fillin(center_rbv,center_val,start,stop,step,**kwargs)
    if kwargs['execute']:
        BL.mda.go(**kwargs)

def slit1A_scribe_marks():
    """
    moves slit1A to the position which corresponds to the scribe marks
    used to check if we have lost steps
    
    Dial values should be 0 on when on the scribe marks
    User values are set to correspond to the center of the beam

    Previously: GoToSlit1AScribe
    """
    main_shutter_close()
    for m in range(9,13):
        caput('29idb:m'+str(m)+'.DVAL',0) 
    
def slit1A_set(H_size,V_size,verbose=True,**kwargs):
    """
    sets the first aperture / slit to the specificed size
    if Hsize / Vsize = inf then goes to maximim 4.5 / 4.5
    **kwargs
        center = slit center, by default => (0,0)

    Previously: SetSlit1A
    """
    kwargs.setdefault('center',(0,0))
    slit_name = 'slit1A'

    if H_size in [inf,nan,None]: 
        H_size=4.5
    if V_size in [inf,nan,None]: 
        V_size=4.5

    size = (H_size,V_size)
    center = kwargs['center']
    slits_set(slit_name,size,center, verbose=verbose)
 
def slit1A_get(verbose=True):
    """
    returns the current (H_size,V_size),(H_center,V_center)
    """
    slit_name = "slit1A"
 
    return  slits_get(slit_name,verbose=verbose)

def slit2B_set(H_size,V_size,verbose=True,**kwargs):
    """
    sets the clean up aperture / slit downstream of the mono to the specificed size
    if Hsize / Vsize = inf then goes to maximim 6 / 8
    
    **kwargs
        center = slit center, by default => (0,0)

    Previously: SetSlit2B
    """   
    kwargs.setdefault('center',(0,0))
    slit_name = "slit2B"

    if H_size in [inf,nan,None]: 
        H_size=6
    if V_size in [inf,nan,None]: 
        V_size=8

    size = (H_size,V_size)
    center = kwargs['center']
    slits_set(slit_name,size,center, verbose=verbose)

def slit2B_set_small(Hsize,Vsize,coef,verbose=True,**kwargs):
    """
    Used to scale the aperature size by coef
        Hsize = Hsize * coef
        Vsize = Vsize * coef
    """
    Hsize = Hsize * coef
    Vsize = Vsize * coef
    print_warning_message("closing Slit-2B down by a factor of", coef, "!!!")
    slit2B_set(Hsize,Vsize,verbose=True,**kwargs)
  
def slit2B_get(verbose=True):
    """
    returns the current (H_size,V_size),(H_center,V_center)
    """
    slit_name = "slit2B"   
    return  slits_get(slit_name,verbose=verbose)

def slit2B_encoders_reset(Hcenter,Vcenter):
    """
    Resetting Slit 2B encoders to 0 for a given position (Hcenter,Vcenter).
    Slit size need to be set to 0.

    Previously: Reset_Slit2B_Encoders
    """
    encoders_reset_zero('slit2B',Hcenter,Vcenter)
    slits_synch('slit2B')


def slit3C_set(size,verbose=True):
    """
    sets slit-3C (ARPES resolution defining slit)

    Previously SetSlit3C
    """
    position=round(slit_3C_fit(size),1)
    caput("29idb:Slit3CFit.A",size,wait=True,timeout=18000)
    
    if verbose:
        print("Slit-3C =",size,"um  -  ( m24 =",position,")")

def slit3C_get(size,verbose=True):
    """
    gets slit-3C (ARPES resolution defining slit)
    returns size, position
    """
    size, position = slit_3C_rbv()
    
    if verbose:
        print("Slit-3C =",size,"um  -  ( m24 =",position,")")

    return size, position

def slit3D_set(V_size,verbose=True,**kwargs):
    """
    sets the clean up aperture / slit downstream of the mono to the specificed size
    if Hsize / Vsize = inf then goes to maximim 6 / 8

    **kwargs
        center = slit center, by default => current value

    Previously: SetSlit3D
    """
    kwargs.setdefault('center',slit3D_get(verbose=False)[1])           
    slit_name = "slit3D"

    if V_size in [inf,nan,None]: 
        V_size=2000

    size = (V_size)
    center = kwargs['center']
    slits_set(slit_name,size,center,verbose=verbose)


def slit3D_get(verbose=True):
    """
    returns the current V_size,V_center
    """
    slit_name = "slit1A"
    (V_size),(V_center) = slits_get(slit_name)
    V_size = round(V_size,3)
    V_center = round(V_center,3)

    return  V_size,V_center

def slit3D_encoders_reset(Hcenter,Vcenter):
    """
    Resetting Slit 2B encoders to 0 for a given position (Hcenter,Vcenter).
    Slit size need to be set to 0.

    Previously: Reset_Slit2B_Encoders
    """
    encoders_reset_zero('slit3D',Hcenter,Vcenter)
    slits_synch('slit3D')

def exit_slit(branch, size, verbose=True):
    """
    verbose used to supress printing

    Previously: SetExitSlit
    """
    if branch == "c":
        slit3C_set(size, verbose)
    elif branch == "d":
        slit3D_set(size, verbose)
    elif branch == "e":
        slit3D_set(size, verbose)





##############################################################################################################
################################             slit fits             ##############################
##############################################################################################################


def aperture_fit(hv,slit_num):
    """
    used close the beamline apertures/slits to only take the center of the beam, 
        i.e. no heat bump (determined emperically by looking at the the shift in energy vs slit position)

    Previously Aperture_Fit
    """
    K=slit_Coef(slit_num)[1]
    sizeH=K[0]+K[1]*hv+K[2]*hv*hv
    sizeV=K[3]+K[4]*hv+K[5]*hv*hv
    return [round(sizeH,3),round(sizeV,3)]

def slit_Coef(slit_num):
    """
    3rd order polynomials coeffients for closing slit1A / slit2B to minimize heat bump effects
    slit_num = 1 / 2 for slit1A / slit2B
    """
    if slit_num == 1:
        pv='29id:k_slit1A'
        #Redshifted x (H):
        H0=2.3325
        H1=-.000936
        H2=2.4e-7
         #Redshifted z (V):
        V0=2.3935
        V1=-.0013442
        V2=3.18e-7
    if slit_num == 2:
        pv='29id:k_slit2B'
        #Redshifted x (H):
        H0=3.61
        H1=-0.00186
        H2=5.2e-7
        #Redshifted z (V):
        V0=6.8075
        V1=-0.003929
        V2=9.5e-7
    K=H0,H1,H2,V0,V1,V2
    return pv,K


def slit_3C_fit(size):
    """
    used to convert slit size to motor position for slit 3C - flexure stage

    empirically determine offline with a camera

    Previously: Slit3C_Fit
    """
    K0=-36.383
    K1=0.16473
    K2=-0.00070276
    K3=8.4346e-06
    K4=-5.6215e-08
    K5=1.8223e-10
    K6=-2.2635e-13
    motor=K0+K1*size+K2*size**2+K3*size**3+K4*size**4+K5*size**5+K6*size**6
    return motor

def slit_3C_rbv():
    """
    used to convert slit motor position to size
    
    Previously: Slit3C_RBV
    """
    position=caget("29idb:m24.RBV")
    K0=299.66
    K1=16.404
    K2=1.5572
    K3=0.14102
    K4=0.0064767
    K5=0.00014501
    K6=1.2617e-06
    size=round(K0+K1*position+K2*position**2+K3*position**3+K4*position**4+K5*position**5+K6*position**6,0)
    return size, position

