"""
Functions for commissioning, alignment and start of the week

"""
from os.path import join, isfile, exists, dirname
from time import sleep

from epics import caget, caput


from ..instruments.files_and_folders import check_run, folder_mda
from ..instruments.scanRecord import scanRecord_run,scanRecord_filepath
from ..instruments.xrays import energy
from ..instruments.m3r import M3R_branch

from ..instruments.IEX_VPU import ID_switch_mode
from ..instruments.diagnostics import diagnostics_all_out, diagnostics_all_in,diode_c,diodeD
from ..instruments.current_amplifiers import current2flux
from ..instruments.slits import set_exit_slit
from ..instruments.Logfile import *
from ..instruments.cameras import *

from ..instruments.ARPES import *
from ..instruments.electron_analyzer import *
from ..instruments.kappa import *


from .quick_plot import fit_mda, mda_1D

##############################################################################################################
################################            setting  the mda folder             ##############################
##############################################################################################################

def staff_init(endstation_name=None,set_folders=True,reset=True):
    """
    endstation_name : sets the scan record and BL
    set_folders: sets the mda and EA folders; default => False
    reset: resets the scanRecord (detectors,triggers...)

    **kwargs:
        xrays: sets global variable; default => True
        BL_mode: 'user' / 'staff' => used for saving, detectors

    Previously: folders_staff
    """
    if endstation_name is None:
        endstation_name = BL.endstation
    
    if endstation_name == 'ARPES':
        ARPES_init(set_folders,reset,BL_mode='staff')
    if endstation_name == 'Kappa':
        Kappa_init(set_folders,reset,BL_mode='staff')



def check_staff_directory(BL, **kwargs):
    """
    Switchs to the staff directory

    **kwargs for folder_ARPES

    Previously: Check_Staff_Directory
    """
    
    ioc = BL.ioc()
    run = check_run()
    
    directory = scanRecord_filepath(ioc)
    current_run = scanRecord_run(ioc)
    
    if directory.find('data_29idb') < 1 or current_run != run:
        print('You are not currently saving in the Staff directory and/or the desired run.')
        # foo=input('\nAre you ready to switch to the '+run+' Staff directory (y or n)? >')  
        # if foo.lower() == 'y' or foo.lower() == 'yes':
        if not (exists(directory)):
            print('Staff directory does not exist for this run.')
            print('Open ipython as 29id to create a new run directory using:\n\tFolder_'+ioc+'(run,"Staff")')
            foo=input('\nPress enter when done >')
            if foo == '': pass   
        print('Switching directory...') 
        if ioc=='ARPES':
            folders_ARPES('Staff',mdaOnly=True,**kwargs)
        elif ioc=='Kappa':
            folders_Kappa('Staff',create_only=False)
        else:
            print('\nFolder not set.')
    else:
        print('Staff directory OK.')
    directory = scanRecord_filepath(ioc)
    print('\nCurrent directory: '+directory)    
    


##############################################################################################################
################################            checking alignment             ##############################
##############################################################################################################

def check_flux(hv=500,ID_mode='RCP',stay=False):
    """
    puts the diode in
    measures flux at energy ID_mode specified
    stay: diode position after the scan

    Previously: CheckFlux
    """
    ID_switch_mode(ID_mode)
    energy(hv)
    diagnostics_all_out(diode_to_stay_in=m3r_branch())
    SR=round(caget("S:SRcurrentAI.VAL"),2)
    if m3r_branch() == "c":
        current_slit=caget('29idb:Slit3CFit.A')
        diode_c('In')
        set_exit_slit(50)
        sleep(10)
        diode=diodeC_read()
    elif m3r_branch() == "d":
        current_slit=caget('29idb:Slit4Vsize.VAL')
        sleep(10)
        diodeD("In")
        set_exit_slit(50)
        diode=caget('29idb:ca14:read')
    flux=current2flux(diode)
    print("\n----- Current on diode   : %.3e" % diode, "A")
    print("----- Corresponding flux: %.3e" % flux, "ph/s \n")
    print("----- Storage ring current: %.2f" % SR, "mA")
    print("\nFlux at hv=500 as off Feb 2019 for RCP and HEG:  ~3.3e-06 A = ~1.5e+11 ph/s")
    
    if not stay:
        diagnostics_all_out()
    set_exit_slit(current_slit)

def check_m0m1(hv=500,stay=True,wire=True,**kwargs):
    """
    Prints Flux in C-branch
    stay = 'yes' => Leaves diagnostics in the beam
    wire ='yes'=> Does wire scans to determine M0M1 alignment
    Logging is automatic: use **kwargs or the optional logging arguments see scanlog() for details
    
    Previously: CheckM0M1
    """
    
    switch_branch('c')
    switch_gratings('HEG')
    print("\nFlux at hv=500 as off Feb 2019:  ~3.3e-06 A = ~1.5e+11 ph/s")
    branch_shutter_open()
    check_flux(hv=hv,stay=stay)
    if wire is not None:
        scan_wire('H',**kwargs)
        scan_wire('V',**kwargs)

def scan_wire(direction,all_diag=True,**kwargs):
    """
    Scans the wires located just downstream of M0/M1, 
         which = 'H' for the horizontal, typically CA2
         which = 'V' for the vertical, typically CA3
    all_diag =True -> AllDiagIn(), otherwise you have to put any diagnostics in by hand
    Logging is automatic: use **kwargs or the optional logging arguments see scanlog() for details    
    **kwargs:
        execute
    
    Previously: WireScan
    """
    kwargs.setdefault('execute',True)
    if all_diag:
        diagnostics_all_in()
    
    
    if direction == 'H':
        name = "H-wire"

    elif direction == 'V':
        name = "V-wire"

    diag = diagnostics_dict()
    pv = "29idb:m"+str(diag["motor"][name])
    print("\n================== "+name+" scan (29idb:ca2):")
    BL.mda.fillin(pv+".VAL",pv+".RBV",1,-17,-30,-0.25,**kwargs)

    if kwargs['execute']:
        BL.mda.go(**kwargs)

    if all_diag:
        diagnostics_all_out()


##############################################################################################################
###########################                    Beam Profile               ######################
##############################################################################################################
def scan_narrow_slit(slit='2V',slit_parameters=[0.25,-2,2,0.5],**kwargs):
    """
        slit = '1V','1H','2V','2H'
        slit_parameters = [SlitSize,start,stop,step]
  
    Typical slit sizes/start/stop/step are (for full range): 
        1H/1V : [0.50, -4.5, 4.5, 0.2]
        2H    : [0.25, -3.0, 3.0, 0.2]
        2V-MEG: [0.25, -4.0, 4.0, 0.2]    
        2V-HEG: [0.50, -8.0, 8.0, 0.2] 

    **kwargs:
        scan_dim = 1
        execute = True

    Previously: Scan_NarrowSlit
    """
    kwargs.setdefault('scan_dim',1)
    kwargs.setdefault('execute',True)

    size,start,stop,step = slit_parameters
      
    direction = slit[1]
    if direction == "V":
        size = (inf, size) 
        center = (0, 0)
    elif direction == "H":
        size = (size, inf) 
        center = (0, 0)

    if slit[0] == '1':
        slit_name = "slit1A"
        slit1A_set(size,center)
        slit2B_set((inf,inf),(0,0))

    elif slit[0] == '2':
        slit_name = "slit2B"
        slit1A_set((3,3),(0,0))
        slit2B_set(size,center)
  
    slits_scan_center(slit_name,direction,start,stop,step,**kwargs)
    if kwargs['execute']:
        BL.mda.go(kwargs['scan_dim'])


def scan_mono_vs_slit(slit='2V',slit_parameters=[0.25,-2,2,0.5],energy_parameters=[470,530,2],**kwargs):
    """
    This can be used to find the center of the resonant beam i.e. the slit value for the most blue shifted curve: 
        slit='1V','1H','2V','2H'
        slit_parameters = [SlitSize,start,stop,step]
        energy_parameters = [eVstart,eVstop,eVstep]= [470,530,2]
        
    Typical slit sizes/start/stop/step are (for full range): 
        1H/1V : [0.50, -4.5, 4.5, 0.2]
        2H    : [0.25, -3.0, 3.0, 0.2]
        2V-MEG: [0.25, -4.0, 4.0, 0.2]    
        2V-HEG: [0.50, -8.0, 8.0, 0.2] 

    Previously: Scan_MonoVsSlit
    """
    hv_start,hv_stop,hv_step = energy_parameters  
    # Filling Scans:
    mono_scan_fillin(hv_start,hv_stop,hv_step)
    scan_narrow_slit(slit='2V',slit_parameters=[0.25,-2,2,0.5],**kwargs)

    # Resetting everybody to normal:
    mono_energy_set((hv_start+hv_stop)/2.0)
    slits_set_BL()


##############################################################################################################
################################            mono alignment             ##############################
##############################################################################################################
def mono_MIR_GRT_find_offset(grating,slit_list,**kwargs):    
    """ 
    Find MIR-GRT offset by scanning 0 order through exit slit
        **kwargs:
            detNum = 15

    Previously: MIR_GRT_Offset
    """
    kwargs.setdefault("detNum",15)

    switch_gratings(grating)
    exit_slit(50)
    diode_c('In')
    slit2B_set(2,0.5,0,0)
    
    #set zero order, scan grating pitch
    for ang in range_up(1.5,5,0.5):
        print("\r")
        mono_zero_order(ang)
        print("Mirror pitch: "+str(ang)+"\r")
        for s in slit_list:
            exit_slit(s)
            print("\r")

            mono_motor_scan_fillin("GRT:P",ang-0.0005*s, ang+0.0005*s ,0.00002*s)
            BL.mda.go(**kwargs)

            sleep(1)
            print("\r")
            grating_ang = fit_mda(mda.fileNum(),kwargs['detNum'],.1,'gauss')
            print("Peak: ",grating_ang)
            print("\r")
            print("-------------------------")

##################### mono slit scans => Energy calibration #####################
def exit_slit_vs_hv(hv,c=3):
    '''
    Adjust slit size to keep reasonable count vs hv
          c=3 for nominal slit2B size
          c=10 for apertured slit2B (e.g 0.25 or 0.5)

    Previously: Mono_Set_Slitvshv
    '''
    slit(hv/100.0*c)


def scan_energy_along_grating(hv,peakBE=84,pts=11,r=0.75,i=1,**kwargs):
    """
    Takes a Slit-2V map for a range of photon energies 
        peakBE=84; fixed mode
        pts = number of pts in slit positions, default: 11
        r = % of range in slit size,  default: 0.75
        i = scanEA time in minute (default:1 for MEG, 2 for HEG?)
        Takes 2.5h with default parameters (i=1)

    Previously: Mono_Scan_Slit2BV  
    """
    grt_density = mono_grating_density_get()
    c=grt_density/1200                # c = 1 for MEG, 2 for HEG

    # Getting EA parameters vs hv:
    PE=200
    
    #Getting Slit-2B parameters
    Vsize=0.25*c
    n=(4.5-hv/500)
    Vstart=-((floor((2*n)/Vsize/2)+1)*Vsize*c)*r
    Vstep=-round(Vstart*2/pts,2)

    #Setting energy:
    energy(hv)
    exit_slit_vs_hv(hv,c=10)

    #Aperturing slit 2V (note that we have the nominal size horizontally)
    (size_rbv,size_val),(center_rbv,center_val) = slits_pvs('slit2B')
    caput(size_val,Vsize)
    
    print("\n--------------- Starting core level vs Slit-2B(V) map ---------------")

    #Take XPS for each Slit-2V position
    for V_center in range_up(Vstart,-Vstart,Vstep):
        caput(center_val,V_center,wait=True,timeout=18000)
        kwargs.update({'comment': "2-V center ="+str(V_center)[0:6]})
        EAlist=["BE",peakBE,PE,17*60*i,1]
        scanEA(EAlist,**kwargs)
    
    #reset the slits
    slits_set_BL()



def scan_mono_energy_drift(start,stop,step,peakBE=84,EF=None,**kwargs):
    '''
    Measure core level (by default Au 4f) in fixed mode vs energy while maintaining resonable count rate
    If EF='y' (by default None) takes the Fermi edge as well.
    kwargs:
        c=3 is the default slit2B size
    
    Used for grating calibration (changing grating_offset and b2 effects this - use grating_offset since b2 was measured)
    
    Previously: Mono_Scan_hv_drift
    '''
    kwargs.setdefault('c','3')

    for hv in range_up(start,stop,step):
        energy(hv)
        c=int(kwargs['c'])
        exit_slit_vs_hv(hv,c)
        kwargs.update({'comment': "Core level @ hv = "+str(hv)})      
        scanEA(["BE",peakBE,200,17*60*1,1],**kwargs) 
        if EF is not None:
            kwargs.update({'comment': "EF @ hv = "+str(hv)})                
            scanEA(["BE",0,200,17*60*10,1],**kwargs) 


    
##############################################################################################################
###########################                    Pinhole Scripts               ######################
##############################################################################################################
def scan_pinhole_full():
    """
    Does a pinhole with .1 mm step with AllDiagIn and then with only branch diode in
    
    Previously: FullPinhole
    """
    diagnostics_all_in()
    scan_pinhole([-3, 3, .5, .1], [-3, 3, .1, .5])
    diagnostics_all_out(diode_stay_in=True)
    scan_pinhole([-3, 3, .5, .1], [-3, 3, .1, .5])


def scan_pinhole(H_list,V_list,**kwargs):
    """
    Sets up the pinhole scan for slit1A
    Make sure that you have the appropriate Diagnostics in can use AllDiagIn()

    H_list = [Hstart, Hstop, Hstep, Hsize] => [-3, 3, .1, .5]
    V_list = [Vstart, Vstop, Vstep, Vsize] => [-3, 3, .1, .5]
    
    *kwargs:
        H_scan_dim => 1
        V_scan_dim => 2
        execute => True

    Previously: Scan_Pinhole_Go, Scan_Pinhole
    """
    kwargs.setdefault('H_scan_dim',1)
    kwargs.setdefault('V_scan_dim',2)
    kwargs.setdefault('execute',True)

    #Synching slits
    slits_synch("slit1A")

    #Getting initial slit size
    (Hsize0,Vsize0),(Hcenter0,Vcenter0) = slit1A_get(verbose=False)

    #Filling in the ScanRecord
    slits_scan_center("slit1A","H",H_list[0],H_list[1],H_list[2],scan_dim=kwargs['H_scan_dim'],execute=False)
    slits_scan_center("slit1A","V",V_list[0],V_list[1],V_list[2],scan_dim=kwargs['V_scan_dim'],execute=False)

    #Set the slit size
    slit1A_set((H_list[2],V_list[2]),(0,0),verbose=True)

    if kwargs['execute']:
        print("\rPinhole scan: ("+str(kwargs['H_scan_dim'])+","+str(kwargs['V_scan_dim'])+") "
        +str(H_list[0])+"/"+str(H_list[1])+"/"+str(H_list[2])+"/"+str(H_list[3])+"/"
        +str(V_list[0])+"/"+str(V_list[1])+"/"+str(V_list[2])+"/"+str(V_list[3]))

        BL.mda.go(max(kwargs['H_scan_dim'],kwargs['V_scan_dim']))
        print("\rNow setting the slits back to:")
        slits_set_BL()

        print("\r")
        print_warning_message("Don't forget to pull all of the diagnostics out.")
   


##############################################################################################################
##################             Slit Calibration & Beam Steering scripts                 ##################
##############################################################################################################


def Notes_from_20191212():
    print(""" looking at diode 3 in scattering chamber, open 3D all the way (2000)
        WARNING: 10% energy detune is enough to see the 2 lobes with 2B but not for 1A => 20% 
        Red shifted  (485eV) = edges give center of the grating
        Blue shifted (510eV) = gaussian center gives center of the beam
        => we want the slit centered on the grating center and the beam centered on the slits/grating
        => 5 urad down = 1 mm negative motion on 2B-V with HEG""")


def procedure_alignment_M1():
    print("""\n-Start with a very red shifted line cut to localize the edges of the grating:
                Set_IDraw(510)
                mono(485)
                Scan_NarrowSlit('2V',[0.25,-8,8,0.5])
                Scan_NarrowSlit('2H',[0.25,-4,4,0.5])
    => Verticaly: The edges/center of the grating define the center of 2V 
    => Horizontaly: The edges are defined by the aperture upstream. 
        As long as the beam makes it through the pipe to the next mirror, we are good. 
        Each M1 pitch will have its own slit-2H zero
        It can be a good idea to check that GRT or M2 translation does not change anything.
\n-Once the slits are centered on the grating, center resonant beam on the grating (zero slit):
                Get_Mirror(1)   # to get current values
                for roll in RangeUp(3,4,0.2):
                    print('\\n')
                    Move_M1('RZ',roll)
                    sleep(5)
                    scan_mono_vs_slit('2V',[0.25,-0.5,0.5,0.25],[490,530,2])
                for pitch in RangeUp(8.5,8.7,0.05):
                    print('\\nMove_M1('RY',pitch)')
                    Move_M1('RY',pitch)
                    sleep(5)
                    scan_mono_vs_slit('2H',[0.25,-0.5,0.5,0.25],[490,530,2])
    => The resonant beam is the slit value for the most blue shifted curve.
    
    WARNING: It is more accurate (and easier) to compare the leading edge of the ID peak 
    than trying to look at the perpendicular direction (slit line cut) because of the 
    asymetry in the beam intensity (uneven carbon strip along the grating?)
""")


    # 2017_3 - HEG after opitmization:  mba_b_0241-0244
    # 2018_1 - HEG before opitmization: mba_b_0005-0008
    
    
def beamsteering_2B(v_position):
    vroll=round(v_position*0.05/0.04,3)
    if v_position<0: v_direction=' +'
    else: v_direction=' -'
    v_steering=v_direction+str(vroll)+' urad'
    return v_steering

def beamsteering_1A(h_position,v_position):
    hrad=round(h_position*10/0.25,0)
    vrad=round(v_position*10/0.25,0)
    if h_position<0: h_direction=' urad outboard'
    else: h_direction=' urad inboard'
    if v_position<0: v_direction=' urad up'
    else: v_direction=' urad down'
    h_steering=str(hrad)+h_direction
    v_steering=str(vrad)+v_direction
    return h_steering,v_steering

    

    

#####################################################################################
### Slit-1A Procedures
#####################################################################################

def check_slit1A(step=0.1):
    """
    Checks that Slit1A is centered on the fixed aperature/theoretical optic axis
        top='m9' -> CA5
        inboard='m10' -> CA4
        
    Scans the top blade - CA4 sees an step function
           top_center = midpoint of motor position for zero current and when current saturates
    Scans the inner blade - CA4 sees box fuction convoluted with gaussian beam position 
           inner_center - midpoint bewteen box edges
           
    SetSlit1A(0,0,inner_center,top_center)
    for m in range(9,13):
        Reset_Motor_User(m,'b',0)
    SyncAllSlits()  
    
    If data makes no sense then Reset_Slit1A_Procedure()
 
    """
   #scanning top-blade
    slit1A_set(8,8,0,0)         # aperture wide open
    caput('29idb:m10.VAL',0)   # Inboard blade centered-ish in the beam
    m=9
    VAL='29idb:m'+str(m)+'.VAL'
    RBV='29idb:m'+str(m)+'.RBV'
    BL.mda.fillin(VAL,RBV,-4,4.0,step)
    BL.mda.go()
    FileNum1  = BL.mda.lastFileNum()

    #scanning inboard-blade
    slit1A_set(8,8,0,0)         # aperture wide open
    caput('29idb:m9.VAL',0)    # top blade centered-ish  in the beam
    m=10
    VAL='29idb:m'+str(m)+'.VAL'
    RBV='29idb:m'+str(m)+'.RBV' 
    BL.mda.fillin(VAL,RBV,-4,4.0,step)
    BL.mda.go()
    FileNum2  = BL.mda.lastFileNum()

    return FileNum1, FileNum2


def procedure_reset_slit1A():
    """
    Prints the procedure for Resetting Slit1A
    """
    print("\n#------- Checking and Resetting the Dial -------")
    print("    GoToSlit1AScribe()")
    print("# if not on the scribe marks tweek the position until the scribe marks are aligned")
    print("    for m in [9,10,11,12]: Reset_Motor_Dial(m,\'b\',0);SyncAllSlits()")
    print("\n#------- Resetting the User -------")
    print("# SetSlit1A by hand such that you have some beam coming through")
    print("    Scan_SlitSize(\"1H\",start,stop,step) #looking at CA4")
    print("    Scan_SlitSize(\"1V\",start,stop,step) #looking at CA4")
    print("# Then SetSlit1A(1Hsize,1Vsize,0,0); where 1Hsize and 1Vsize are the size where CA4 = 0")
    print("    for m in [9,10,11,12]: Reset_Motor_User(m,'b',0);SyncAllSlits()")
    
  
def check_ID_steering(hv=2000):
    """
    Scans Slit1A center (set to a (0.25,0.25) pinhole) while looking at the back blade:
        - slit center vs fixed aperture: given by the position of the edges
        - beam center vs fixed aperture: given by the position of the bump in the middle
    """
    slit1A_set(0.25,0.25,0,0)
    ID_SP_set(hv)
    scan_slit_center('1H',-3,3,0.1)
    scan_slit_center('1V',-3,3,0.1)
        
def scan_slit_center(slit,start,stop,step,size=0.25):
    """
    slit = '1H','1V','2H' or '2V'

    Previously: Scan_SlitCenter
    """
    if slit[0] == '1':
        slit_name = "slit1A"
    elif slit[0] == '2':
        slit_name = "slit2B"
    direction = slit[1]
    slits_scan_center(slit_name,direction,start,stop,step,size=size)


def scan_slit_size(slit,start,stop,step,center=(0,0)):
    """
    slit = '1H','1V','2H' or '2V'

    Previously: Scan_SlitSize
    """
    if slit[0] == '1':
        slit_name = "slit1A"
    elif slit[0] == '2':
        slit_name = "slit2B"
    direction = slit[1]
    slits_scan_center(slit_name,direction,start,stop,step,center=center)



#####################################################################################
# Checking the beam steering uses the gas-cell to see if the beam is centered on the grating
# and that the slits are centered
#####################################################################################

def ID_beam_profile(grt,slit_list,c_slit=1,c_energy=1,**kwargs):
    """
        Makes a nice 2D image of the energy distribution of the beam across the grating at ID=500
        Does NOT put the diagnostics into the beam you need to run the following if you haven't already (AllDiagOut(); DiodeCIn())
        SlitList=["2H","2V","1H","1V"]
        c_slit  = scaling of step size (c=1   Slit-1: step = 0.25. Slit-2: step = 0.5)
        c_energy = scaling of mono step size ( c=1   eV step = 2)
        with c_slit=1 and c_energy = 1   Each slit ~ 1:10 min

        Previously
    """
    switch_gratings(grt)
    grt_density = mono_grating_density_get()
    c=grt_density/1200
    ID=500
    eVstart,eVstop,eVstep=460,540,4
    for slit in slit_list:
        if slit=="1H":
            #Hsize,Vsize,Hstart,Hstop,Hstep = 0.50,2,-2,2,0.25*c_slit      # => 35 min
            #MonoVsSlit1AH_Go(ID,eVstart,eVstop,eVstep*c_energy,Hstart,Hstop,Hstep,Hsize,Vsize,scanIOC,**kwargs)
            scan_mono_vs_slit('1H',[0.5,-2,2,0.25*c_slit],[eVstart,eVstop,eVstep],comment='Mono/Slit - 1H')
        elif slit == "1V":
            #Hsize,Vsize,Vstart,Vstop,Vstep = 2,0.50,-2,2,0.25*c_slit
            #MonoVsSlit1AV_Go(ID,eVstart,eVstop,eVstep*c_energy,Vstart,Vstop,Vstep,Hsize,Vsize,scanIOC,**kwargs)
            scan_mono_vs_slit('1V',[0.5,-2,2,0.25*c_slit],[eVstart,eVstop,eVstep],comment='Mono/Slit - 1V')
        elif slit =="2H":
            #Hsize,Vsize,Hstart,Hstop,Hstep = 0.50,8,-3,3,0.5*c_slit
            #MonoVsSlit2BH_Go(ID,eVstart,eVstop,eVstep*c_energy,Hstart,Hstop,Hstep,Hsize,Vsize,scanIOC,**kwargs)
            scan_mono_vs_slit('2H',[0.5,-3,3,0.5*c_slit],[eVstart,eVstop,eVstep],comment='Mono/Slit - 2H')
        elif slit =="2V":
            #Hsize,Vsize,Vstart,Vstop,Vstep = 6,0.25*c,-4*c,4*c,0.5*c_slit
            #MonoVsSlit2BV_Go(ID,eVstart,eVstop,eVstep*c_energy,Vstart,Vstop,Vstep,Hsize,Vsize,scanIOC,**kwargs)
            scan_mono_vs_slit('2V',[0.5,-4,4,0.5*c_slit],[eVstart,eVstop,eVstep],comment='Mono/Slit - 2V')


def CheckAllSlits_long(hv_list=[500],slit_list=["1H","1V","2H","2V"],**kwargs):
    """
    For each photon energy in hvList, takes 3 slit curves @ mono below / resonance / above
    For given SlitList
    For both gratings
    """
    for grt in ["HEG","MEG"]:    # One full loop for a given grating and 3 energies takes 5h for average 20
        switch_gratings(grt)
        for hv in hv_list:
            slits_set_BL(hv)
            step=hv/100.0
            start=hv-step
            stop=hv+step
            for slit in slit_list:
                for hv in range(start,stop,step):
                    print("\r")
                    mono_energy_set(hv)
                    check_ID_beam_steering(slit)


def CheckSlitCalibration(slit_list,BL=500,hvList=[485,510],scanIOC=None,**kwargs):
    """
    Slit scan for red shifted and blue shifted mono values
    used to determine in the beam is centered on the grating (gaussian)
    and if the slits are centered on the grating (humped)
                 hv=500; hvList=[485,510]
                 hv=1500;hvList=[hv*0.97,hv*1.01]
    Note: sets the exit slit to 50
    """
    if scanIOC is None:
        scanIOC=BL_ioc()
    exit_slit(50)
    slits_set_BL(BL)
    for hv in hvList:
        mono_energy_set(hv)
        for slit in slit_list:
            check_ID_beam_steering(slit)
        print("\n")

def check_ID_beam_steering(slit):   #here
    """
    checks the ID beam steering by scanning the slit with the ID detuned
    if there is carbon on the mirror you might need to do the full ID_beam_profile to
    find the red shifted beam

    slit = '1V','1H','2V','2H'

    Previously: CheckBeamPosition
    """
             
    grt_density = mono_grating_density_get()    # can be plotted directly on dview using the buffer. That should
    c=grt_density/1200                # be fairly quick and not require any data loading/analysis
    
    slit1A_set((3.5,3.5),(0,0),verbose=False)
    slit2B_set((6.0,8.0),(0,0),verbose=False)        # Open up all the slits to make sure we scan the full beam
    
    if slit == "1H":
        size,start,stop,step = 0.50,-4.5,4.5,0.2
    elif slit == "1V":
        size,start,stop,step = 0.50,-4.5,4.5,0.2
    elif slit == "2H":
        size,start,stop,step = 0.25,-8.0,8.0,0.2
    elif slit == "2V":
        size,start,stop,step = 0.25*c,-4*c,4*c,0.2*c

    scan_slit_center(slit,start,stop,step,size=size)    
    slits_set_BL()


##############################################################################################################
##################                 Commissioning paper curves                     ##################
##############################################################################################################
def FermiEdges(Energy,Slit):
    EF_Table={}
    EF_Table[500]  = {10:20,  20:20,  50:20,  100:20,  200:20}
    EF_Table[1000] = {10:50,  20:50,  50:50,  100:50,  200:50}
    EF_Table[1500] = {10:100, 20:100, 50:100, 100:100, 200:100}
    PE = EF_Table[Energy][Slit]
    return PE




def QP_curves(ID_energy,ID_mode_list,hv_start,hv_stop,hv_step):
    """
    """
    exit_slit(200)
    #Switch_Grating("MEG")
    #print "\r"
    #print "****************  QP OFF  ****************"
    #for mode in list_mode:
    #    Switch_IDMode(mode)
    #    SetID_Raw(hv)
    #    SetSlit_BL()
    #    SetMono(250)
    #    Scan_Mono(1,start,stop,step)
    #    Scan_Go("b",1)
    #print "\r"
    #print "****************  QP ON  ****************"
    #Switch_IDQP("on")
    for ID_mode in ID_mode_list:
        polarization(ID_mode)
        ID_SP_set(ID_energy)
        slits_set_BL()()
        mono_energy_set(250)
        mono_scan_fillin(hv_start,hv_stop,hv_step)
        BL.mda.go()
    #Switch_IDQP("off")
    #Switch_Grating("HEG")
    #Switch_IDMode("RCP")


##############################################################################################################
################################          ID calibration scripts        ##############################
##############################################################################################################

######  Energy range follows ID ########

def ID_calibration_scan(ID_start,ID_stop,ID_step,bandwidth=10,QP=None,Harm=1,scanIOC=None):
    """
    Sets the ID_SP and scans the mono:
        start,stop,step are for ID
    if QP is not None adjusts the mono range to offset accordingly
    !!! DOES NOT CHANGE QP VALUE !!!!
    !!! Doesn't put diodes in !!!
    """
    
    print("""
             !!! DOES NOT CHANGE QP VALUE !!!!
             !!! Doesn't put diodes in !!!""")
    
    logfile_name_set(BL.endstation,filename_suffix='IDCalibration')

    #Getting mono max range based on grating
    grt_density = mono_grating_density_get()
    c=grt_density/1200            # c = 1 for MEG, 2 for HEG
    if c == 1: 
        maxhv = 3000
    elif c == 2: 
        maxhv = 2000
   
    log_print(comment="====== Starting Mono Scan vs ID ======")

    for ID_energy in range_up(ID_start,ID_stop,ID_step):
        print("\n------------------  ID-SP @ "+str(ID_energy)+"   ------------------")
        mono_energy_set(ID_energy)
        ID_SP_set(ID_energy)
        slits_set_BL()
        branch = m3r_branch()
        if branch == 'd':
            m3r_align()
        if QP is None:
            start_hv=min(ID_energy-ID_energy/(bandwidth*1.0)*Harm,maxhv)  # FR update range 11/20/2019
            stop_hv=min(ID_energy+ID_energy/(bandwidth*2.0)*Harm,maxhv)
            step_hv=round(ID_energy/300.0,1)
#            start_hv=min(ID_energy-ID_energy/(bandwidth*2.0)*Harm,maxhv)  # shift the window by doing -BW*2/+BW*1 instead of -BW*1'/'+BW*2
#            stop_hv=min(ID_energy+ID_energy/(bandwidth*1.0)*Harm,maxhv)
#            step_hv=round(ID_energy/500.0,1)
        else:
            start_hv=min(ID_energy-ID_energy/(bandwidth*2.5)*Harm,maxhv)
            stop_hv=min(ID_energy+ID_energy/(bandwidth*0.7)*Harm,maxhv)
            step_hv=round(ID_energy/300.0,1)

        mono_scan_fillin(start_hv,stop_hv,step_hv)
        BL.mda.go()
        sleep(1)

        FileNum  = BL.mda.lastFileNum
        if ID_energy == ID_start:
            FirstFileNum = FileNum     # Record First Scan Number
    
    LastFileNum = FileNum              # Record Last Scan Number

    print("Use:  update_id_dict("+str(FirstFileNum)+","+str(LastFileNum)+"ignore=[] ,update_file,path=None,prefix='ARPES',q=True)")




def ID_calibration_mode(ID_mode,start=None,stop=None,step=25,Harm=1):
    '''
    Runs full calibration with a 25 eV step (by default) for a given mode:
            which = 'RCP','LCP', 'V','H'
    Better to run each mode in a separated cell (print statement at the end helps with calibration) 
    '''
    branch = m3r_branch()
    diagnostics_all_out(diode_stay_in=True)

    polarization(ID_mode)

    grt = mono_grating_get() 

    QP_ratio = ID_QP_ratio_get()
    
    hv_min,hv_max = energy_range_min_max()
    if start == None:
        start = hv_min
    if stop == None:
        stop=hv_max-50

    if QP_ratio <100:
        stop=min(stop,1500)
        QP_ratio=1
    else: 
        QP_ratio=None

    ID_calibration_scan(start,stop,step,QP=QP_ratio,Harm=Harm)


##############################################################################################################
##############################             Beam Motion            ##############################
##############################################################################################################
def scan_cam_motor(motor_name,start,stop,step,cam_num,ExposureTime):
    """   
    motor_name = 'x','y','z'...
    cam_num = 1,2,3... uses cam_pv to get the pv

    Previously: Pixel_Calibration
    """    
    cam_scan_setup(cam_num,ADtype='TIFF')

    if BL.branch == 'ARPES':
        ARPES_Motors.scan(motor_name,start,stop,step)
    if BL.branch == 'Kappa':
        Kappa_Motors.scan(motor_name,start,stop,step)
    
    cam_live()



def scan_cam_hv(start,stop,step,cam_num,**kwargs):
    """   
    takes and image as a function of photon energy
    **kwargs:
        ExposureTime
    Previously: BeamMotion_Vs_BL_Go
    """
    cam_scan_setup(cam_num,ADtype='TIFF')
    scanhv(start,stop,step,**kwargs)

    cam_live()



def scan_cam_mono_zero_order(start,stop,step,cam_num,iterations=1,**kwargs):
    """
    Previously: BeamMotion_Vs_MonoZero_Go
    """
    i = 0
    while i < iterations:
        for angle in range(start,stop+step,step):
            mono_zero_order(angle)
            sleep(5)
            cam_snap(cam_num,ADtype='TIFF',**kwargs)
        i+=1
    
    cam_live()

def scan_cam_mono_pitch(motor,angle,delta,cam_num,iterations=1,**kwargs):
    """
    takes image at zero order (alpha = beta = angle)
    takes image at zeror order with delta 

    motor = 'grating' or 'mirror
        if 'grating': 
            alpha = angle
            beta = angle + delta 
        if 'mirror': 
            alpha = angle + delta 
            beta = angle

    Previously:BeamMotion_Vs_MirPitch_Go
    """
    if motor == 'grating': 
        alpha = angle
        beta = angle + delta 
    if motor == 'mirror': 
        alpha = angle + delta 
        beta = angle
    i=0
    while i<iterations:
        mono_angles_set(angle,angle)
        sleep(5)
        cam_snap(cam_num,ADtype='TIFF',**kwargs)
        
        mono_angles_set(alpha,beta)
        sleep(5)
        cam_snap(cam_num,ADtype='TIFF',**kwargs)
        i+=1

    cam_live()



